/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */


import java.awt.Color;
import main.airfoil.ConstantAirfoil;
import main.atmosphere.Atmosphere;
import main.atmosphere.ConstantAtmosphere;
import main.inducedvelocity.ConstantInducedVelocity;
import main.inducedvelocity.ShaydakovInducedVelocity;
import main.rotor.GlauertRotor;
import org.apache.commons.math3.util.FastMath;
import org.jfree.chart.ChartFrame;
import org.jfree.chart.ChartTheme;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.StandardChartTheme;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * Glauert rotor test. Display one blade thrust and two blades in plane force chart
 */
public class BladeForcesOverAzimut {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {

        // Define rotor
        ConstantAirfoil bladeProfile = new ConstantAirfoil(5.7, 0.011, 0, 0);
        ConstantInducedVelocity inducedVelocity = new ShaydakovInducedVelocity();
        Atmosphere atmosphere = new ConstantAtmosphere(1.225, 0, 0);
        GlauertRotor rotor = new GlauertRotor(2, 4, 0.2, 0, Math.toRadians(0), 0,
                0.97, 64, 64.04, bladeProfile, inducedVelocity);

        // Simulate in autorotative mode : 80 km/h, 10 degres incidence, 
        // 3 degres blade pitch, 0 pitch and roll rate
        rotor.computeState(80 / 3.6, Math.toRadians(10), Math.toRadians(3), 0, 0,
                atmosphere);

        // Compute periodic force
        int n = 200;
        double step = 2 * FastMath.PI / (n - 1);
        double[] angle = new double[n];
        double[] thrust1B = new double[n];
        double[] thrust2B = new double[n];
        double[] H = new double[n];
        double[] Y = new double[n];

        for (int i = 0; i < n; i++) {
            angle[i] = i * step;
            thrust1B[i] = rotor.getThrust() / 2
                    + rotor.getPeriodicThrust1S() * FastMath.sin(angle[i])
                    + rotor.getPeriodicThrust1C() * FastMath.cos(angle[i])
                    + rotor.getPeriodicThrust2S() * FastMath.sin(2 * angle[i])
                    + rotor.getPeriodicThrust2C() * FastMath.cos(2 * angle[i]);
            thrust2B[i] = rotor.getThrust()
                    + 2 * (rotor.getPeriodicThrust2S() * FastMath.sin(2 * angle[i])
                    + rotor.getPeriodicThrust2C() * FastMath.cos(2 * angle[i]));
            H[i] = rotor.getTotalRearForce()
                    + 2 * (+rotor.getProfileDragPeriodicRearForce2C() * FastMath.cos(2 * angle[i])
                    + rotor.getInducedPeriodicRearForce2S() * FastMath.sin(2 * angle[i])
                    + rotor.getInducedPeriodicRearForce2C() * FastMath.cos(2 * angle[i]));
            Y[i] = rotor.getLateralForce()
                    + 2 * (+rotor.getProfileDragPeriodicLateralForce2S() * FastMath.sin(2 * angle[i])
                    + rotor.getInducedPeriodicLateralForce2S() * FastMath.sin(2 * angle[i])
                    + rotor.getInducedPeriodicLateralForce2C() * FastMath.cos(2 * angle[i]));
            H[i] = (H[i] - rotor.getLongitudinalFlapping() * thrust2B[i]);
            Y[i] = (Y[i] - rotor.getLateralFlapping() * thrust2B[i]);
            H[i] = H[i] / 10;
            Y[i] = Y[i] / 10;
            thrust1B[i] = thrust1B[i] / 10;
            thrust2B[i] = thrust2B[i] / 10;
            angle[i] = FastMath.toDegrees(angle[i]);
        }

        // Thrust one blade chart ----------------------------------------------
        final XYSeries Thrust1B = new XYSeries("One blade");
        for (int i = 0; i < angle.length; i++) {
            Thrust1B.add(angle[i], thrust1B[i]);
        }
        // Collection
        final XYSeriesCollection Thrust1BData = new XYSeriesCollection();
        Thrust1BData.addSeries(Thrust1B);

        // Plot
        final NumberAxis xThrust1BAxis = new NumberAxis("deg");
        final NumberAxis yThrust1BAxis = new NumberAxis("daN");
        yThrust1BAxis.setAutoRangeIncludesZero(false);
        final XYItemRenderer thrust1BRenderer = new XYLineAndShapeRenderer(true, false);
        thrust1BRenderer.setBaseToolTipGenerator(new StandardXYToolTipGenerator());
        final XYPlot thrust1BPlot = new XYPlot(Thrust1BData, xThrust1BAxis,
                yThrust1BAxis, thrust1BRenderer);

        // Chart
        final JFreeChart thrust1BChart = new JFreeChart("",
                JFreeChart.DEFAULT_TITLE_FONT, thrust1BPlot, false);
        ChartTheme incidenceAndDraChartTheme = new StandardChartTheme("JFree");
        incidenceAndDraChartTheme.apply(thrust1BChart);

        // In plane force ------------------------------------------------------
        final XYSeries inplane1 = new XYSeries("inplane1");
        final XYSeries inplane2 = new XYSeries("inplane2");
        for (int i = 15; i < 66; i++) {
            inplane1.add(Y[i], H[i]);
        }
        for (int i = 0; i < 16; i++) {
            inplane2.add(Y[i], H[i]);
        }
        for (int i = 65; i < 100; i++) {
            inplane2.add(Y[i], H[i]);
        }

        // Collection
        final XYSeriesCollection inplaneData = new XYSeriesCollection();
        inplaneData.addSeries(inplane1);
        inplaneData.addSeries(inplane2);

        // Plot
        final NumberAxis xinplaneAxis = new NumberAxis("Y (daN)");
        xinplaneAxis.setAutoRangeIncludesZero(false);
        final NumberAxis yinplaneAxis = new NumberAxis("H (daN)");
        yinplaneAxis.setAutoRangeIncludesZero(false);
        final XYPlot inplanePlot = new XYPlot(inplaneData, xinplaneAxis,
                yinplaneAxis, null);
        
        // Renderer
        final XYItemRenderer inplaneRenderer = new XYLineAndShapeRenderer(true, false);
        inplaneRenderer.setBaseToolTipGenerator(new StandardXYToolTipGenerator());
        inplaneRenderer.setSeriesPaint(1, Color.RED);
        
        // Chart
        final JFreeChart inplaneChart = new JFreeChart("",
                JFreeChart.DEFAULT_TITLE_FONT, inplanePlot, false);
        inplaneChart.removeLegend();
        ChartTheme inplaneTheme = new StandardChartTheme("JFree");
        inplaneTheme.apply(inplaneChart);
        inplanePlot.setRenderer(inplaneRenderer);

        // Show chart
        // thrust1BChart or inplaneChart
        ChartFrame frame = new ChartFrame("", inplaneChart);
        frame.pack();
        frame.setVisible(true);
    }
}
