/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */


import main.airfoil.ConstantAirfoil;
import main.atmosphere.Atmosphere;
import main.atmosphere.ConstantAtmosphere;
import main.inducedvelocity.ConstantInducedVelocity;
import main.inducedvelocity.ShaydakovInducedVelocity;
import main.rotor.GlauertRotor;
import org.apache.commons.math3.util.FastMath;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartFrame;
import org.jfree.chart.JFreeChart;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * Glauert rotor test. Display balde force over radius for 90 and 270 deg azimuth.
 *
 * @author J. Fourcade
 */
public class BladeForcesOverRadius {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {

        // Define rotor
        ConstantAirfoil bladeProfile = new ConstantAirfoil(5.7, 0.011,0, 0);
        ConstantInducedVelocity inducedVelocity = new ShaydakovInducedVelocity();
        Atmosphere atmosphere = new ConstantAtmosphere(1.225, 0, 0);
        GlauertRotor rotor = new GlauertRotor(2, 4, 0.2, 0, Math.toRadians(0), 0, 
                0.97, 64, 64.04, bladeProfile, inducedVelocity);

        // Simulate in autorotative mode : 80 km/h, 10 degres incidence, 
        // 3 degres blade pitch, 0 pitch and roll rate
        rotor.computeState(80 / 3.6, Math.toRadians(10), Math.toRadians(3), 0, 0,
                atmosphere);

        // Compute periodic force
        int n = 200;
        double step = 2 * FastMath.PI / (n - 1);
        double[] x = new double[n];
        double[] thrust90 = new double[n];
        double[] thrust270 = new double[n];

        for (int i = 0; i < n; i++) {
            x[i] = i * step;
            thrust90[i] = rotor.getThrust() / 2
                    + rotor.getPeriodicThrust1S() * FastMath.sin(x[i])
                    + rotor.getPeriodicThrust1C() * FastMath.cos(x[i])
                    + rotor.getPeriodicThrust2S() * FastMath.sin(2 * x[i])
                    + rotor.getPeriodicThrust2C() * FastMath.cos(2 * x[i]);
            thrust270[i] = rotor.getThrust()
                    + 2 * (rotor.getPeriodicThrust2S() * FastMath.sin(2 * x[i])
                    + rotor.getPeriodicThrust2C() * FastMath.cos(2 * x[i]));
            thrust90[i] = thrust90[i] / 10;
            thrust270[i] = thrust270[i] / 10;
        }

        // Compute incidence
        final double azimuth = 3*FastMath.PI / 2;
        n = 50;
        step = 1.0 / (n - 1);
        double r[] = new double[n];
        double aoa[] = new double[n];
        for (int i = 0; i < n; i++) {
            r[i] = i * step;
            aoa[i] = rotor.computeAoAForGivenRadialPosition(r[i], azimuth);
            if (aoa[i] > FastMath.toRadians(12) || aoa[i] < 0) {
                aoa[i] = 0;
            } else {
                aoa[i] = FastMath.toDegrees(aoa[i]);
            }
        }
        
        // Thrust one blade chart ------------------------------------------------------------
        final XYSeries Thrust90 = new XYSeries("90 deg");
        final XYSeries Thrust270 = new XYSeries("270 deg");
        for(int i=0;i<x.length;i++) {
           Thrust90.add(x[i],thrust90[i]); 
           Thrust270.add(x[i],thrust270[i]);  
        }
        // Serie collection
        final XYSeriesCollection ThrustData = new XYSeriesCollection();
        ThrustData.addSeries(Thrust90);
        ThrustData.addSeries(Thrust270);

        // Create Chart
        final JFreeChart ThrustChart = ChartFactory.createXYLineChart(
                "Rotor thrust one blade",
                "Azimut", // x axis label
                "daN", // y axis label
                ThrustData);
        
        // Show chart
        ChartFrame frame = new ChartFrame("", ThrustChart);
        frame.pack();
        frame.setVisible(true);
    }
}
