/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */


import main.airfoil.ConstantAirfoil;
import main.atmosphere.Atmosphere;
import main.atmosphere.ConstantAtmosphere;
import main.inducedvelocity.ConstantInducedVelocity;
import main.inducedvelocity.ShaydakovInducedVelocity;
import main.rotor.GlauertRotor;
import org.apache.commons.math3.util.FastMath;
import org.jfree.chart.ChartFrame;
import org.jfree.chart.ChartTheme;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.StandardChartTheme;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * Glauert rotor test. Display rotor torque of one blabe over azimuth
 *
 * @author J. Fourcade
 */
public class BladeTorqueOverAzimut {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {

        // Define rotor
        // Define rotor
        ConstantAirfoil bladeProfile = new ConstantAirfoil(5.7, 0.011,0, 0);
        ConstantInducedVelocity inducedVelocity = new ShaydakovInducedVelocity();
        Atmosphere atmosphere = new ConstantAtmosphere(1.225, 0, 0);
        GlauertRotor rotor = new GlauertRotor(2, 4, 0.2, 0, Math.toRadians(0), 0, 
                0.97, 64, 64.04, bladeProfile, inducedVelocity);

        // Simulate in autorotative mode : 80 km/h, 10 degres incidence, 
        // 3 degres blade pitch, 0 pitch rate
        rotor.computeState(80 / 3.6, Math.toRadians(10), Math.toRadians(3), 0, 0,
                atmosphere);

        // Compute periodic torque
        int n = 200;
        double step = 2 * FastMath.PI / (n - 1);
        double angle[] = new double[n];
        double[] torque1B = new double[n];
        double[] torque2B = new double[n];

        for (int i = 0; i < n; i++) {
            angle[i] = i * step;
            torque1B[i] = rotor.getTotalTorque()/ 2
                    + rotor.getProfileDragPeriodicTorque1S() * FastMath.sin(angle[i])
                    + rotor.getInducedPeriodicTorque1C() * FastMath.cos(angle[i])
                    + rotor.getInducedPeriodicTorque2S() * FastMath.sin(2 * angle[i])
                    + rotor.getProfileDragPeriodicTorque2C() * FastMath.cos(2 * angle[i]) ;         
            torque2B[i] = rotor.getTotalTorque()
                    + 2 * (rotor.getInducedPeriodicTorque2S() * FastMath.sin(2 * angle[i])
                    + rotor.getProfileDragPeriodicTorque2C() * FastMath.cos(2 * angle[i]));
            torque1B[i] = torque1B[i] / 10;
            torque2B[i] = torque2B[i] / 10;
            angle[i] = FastMath.toDegrees(angle[i]);
        }
        
        // Thrust one blade chart ------------------------------------------------------------
        final XYSeries Torque1B = new XYSeries("One blade");
        for(int i=0;i<angle.length;i++) {
           Torque1B.add(angle[i],torque1B[i]); 
        }
        // Collection
        final XYSeriesCollection Torque1BData = new XYSeriesCollection();
        Torque1BData.addSeries(Torque1B);
        
        // Plot
        final NumberAxis xTorque1BAxis = new NumberAxis("deg");
        xTorque1BAxis.setAutoRangeIncludesZero(false);
        final NumberAxis yTorque1BAxis = new NumberAxis("daN.m");
        yTorque1BAxis.setAutoRangeIncludesZero(false);
        final XYItemRenderer torque1BRenderer = new XYLineAndShapeRenderer(true, false);
        torque1BRenderer.setBaseToolTipGenerator(new StandardXYToolTipGenerator());
        final XYPlot thrust1BPlot = new XYPlot(Torque1BData, xTorque1BAxis,
                yTorque1BAxis, torque1BRenderer);
        
        // Chart
        final JFreeChart torque1BChart = new JFreeChart("",
                JFreeChart.DEFAULT_TITLE_FONT, thrust1BPlot, false);
        ChartTheme torqueTheme = new StandardChartTheme("JFree");
        torqueTheme.apply(torque1BChart);
   
        // Show chart
        ChartFrame frame = new ChartFrame("", torque1BChart);
        frame.pack();
        frame.setVisible(true);
    }
}
