
/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty and the software's author, the holder of the
 * economic rights, and the successive licensors  have only  limited
 * liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading, using, modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean that it is complicated to manipulate, and that also
 * therefore means that it is reserved for developers and experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and, more generally,to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
import java.awt.Color;
import java.awt.Dimension;
import main.airfoil.ConstantAirfoil;
import main.atmosphere.Atmosphere;
import main.atmosphere.DensityAtmosphere;
import main.inducedvelocity.ConstantInducedVelocity;
import main.inducedvelocity.ShaydakovInducedVelocity;
import main.rotor.GlauertRotor;
import main.utils.PolynomialFitting;
import org.apache.commons.math3.analysis.MultivariateFunction;
import org.apache.commons.math3.optim.InitialGuess;
import org.apache.commons.math3.optim.MaxEval;
import org.apache.commons.math3.optim.PointValuePair;
import org.apache.commons.math3.optim.nonlinear.scalar.GoalType;
import org.apache.commons.math3.optim.nonlinear.scalar.ObjectiveFunction;
import org.apache.commons.math3.optim.nonlinear.scalar.noderiv.NelderMeadSimplex;
import org.apache.commons.math3.optim.nonlinear.scalar.noderiv.SimplexOptimizer;
import org.apache.commons.math3.util.FastMath;
import org.jfree.chart.ChartFrame;
import org.jfree.chart.ChartTheme;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.StandardChartTheme;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.plot.CombinedDomainXYPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * KD1 gyro computation (Naca report 591 and 600)
 * 
 * @author J. Fourcade
 */
public class KD1 {

    // Define KD1 rotor 
    private final static double a = 5.86;
    private final static double Cm = -0.056;
    private final static double xac = 0.242;
    private final static int b = 3;
    private final static double R = 6.096;
    private final static double c = 0.3048;
    private final static double xcg = 0.28;
    private final static double thetaTW = 0;
    private final static double GJ = 10706.27; // Value from NACA 600
    //private final static double GJ = 12700; // Value ajusted for best deformation
    private final static double B = 0.975;
    private final static double I = 237.268;
    private final static double weight = 9344.45;
    private final static double bladePitch = 0.096;
    private final static Atmosphere atmosphere = new DensityAtmosphere(1.191);
    private final static ConstantInducedVelocity inducedVelocity = new ShaydakovInducedVelocity();

    final private SimplexOptimizer optimizer;
    final private ZeroFunction f;
    private double velocity;
    private double omega;
    private GlauertRotor rotor;

    /**
     * Constructor
     */
    public KD1() {

        // Optimizer
        optimizer = new SimplexOptimizer(1e-8, 1e-10);

        // Objective function
        f = new ZeroFunction();
    }

    /**
     * Compute rotor no-feathering disk incidence and blade profile drag coefficient
     * so that rotor lift equal gyro weight an total torque is zero
     * 
     * @param velocity : airspeed of the gyro (m/s)
     * @param omega : rotor speed (rad/s) 
     * @param startPoint : sart point (indidence and profile drag)
     * 
     * @return incidence (rad) and profile drag
     */
    public PointValuePair compute(double velocity, double omega,
            double[] startPoint) {

        this.velocity = velocity;
        this.omega = omega;

        // Optimize
        return optimizer.optimize(new ObjectiveFunction(f),
                new NelderMeadSimplex(new double[]{FastMath.toRadians(5),
                    FastMath.toRadians(5)}),
                GoalType.MINIMIZE,
                new InitialGuess(startPoint),
                new MaxEval(500));
    }

    private class ZeroFunction implements MultivariateFunction {

        @Override
        public double value(double[] doubles) {
            final double incidence = doubles[0];
            final double delta = doubles[1];

            ConstantAirfoil bladeProfile = new ConstantAirfoil(a, delta, Cm, xac);
            rotor = new GlauertRotor(b, R, c, xcg, thetaTW, GJ, B, I, I,
                    bladeProfile, inducedVelocity);

            // Compute thrust state
            rotor.computeState(velocity, incidence, bladePitch, 0, 0, omega, atmosphere);

            // Lift equation
            final double eq1 = rotor.getLift() - weight;

            // Torque equation
            final double eq2 = rotor.getTotalTorque();

            // Retrun zero function
            return FastMath.pow(eq1, 2) + FastMath.pow(eq2, 2);
        }
    }

    /**
     * Get rotor
     * 
     * @return rotor
     */
    public GlauertRotor getRotor() {
        return rotor;
    }

    /**
     *
     * @param args
     */
    public static void main(String[] args) {

        // Rotor speed tabulated values (rpm and mph)
        final double[] rotorSpeedTab = {192.889, 194.222, 196, 198.222, 202.222,
            207.111, 212, 216.444};
        final double[] velocityTab = {33.1111, 44.8889, 54.6667, 64.6667, 74.8889,
            84.8889, 93.3333, 99.7778};

        // Least square interpolation of rotor speed
        PolynomialFitting rotorSpeedFunction = new PolynomialFitting(4, rotorSpeedTab,
                velocityTab);

        // KD1 class computation
        KD1 kd1 = new KD1();

        // Flapping series
        final XYSeries a0 = new XYSeries("a0");
        final XYSeries a1 = new XYSeries("a1");
        final XYSeries b1 = new XYSeries("b1");
        final XYSeries a2 = new XYSeries("a2");
        final XYSeries b2 = new XYSeries("b2");
        final XYSeries a0Mes = new XYSeries("a0Mes");
        final XYSeries a1Mes = new XYSeries("a1Mes");
        final XYSeries b1Mes = new XYSeries("b1Mes");
        final XYSeries a2Mes = new XYSeries("a2Mes");
        final XYSeries b2Mes = new XYSeries("b2Mes");
        a0Mes.add(0.124798, 8.4397);
        a0Mes.add(0.158669, 8.14246);
        a0Mes.add(0.187702, 7.79648);
        a0Mes.add(0.216734, 7.72411);
        a0Mes.add(0.245161, 7.53012);
        a0Mes.add(0.272984, 6.49707);
        a0Mes.add(0.297782, 6.9779);
        a0Mes.add(0.316532, 6.26083);
        a1Mes.add(0.126008, 0.930785);
        a1Mes.add(0.159274, 1.53339);
        a1Mes.add(0.187702, 1.63733);
        a1Mes.add(0.216734, 1.49807);
        a1Mes.add(0.245161, 1.7905);
        a1Mes.add(0.272379, 1.23776);
        a1Mes.add(0.297782, 1.39635);
        a1Mes.add(0.316532, 1.36633);
        a2Mes.add(0.124798, 0.0795441);
        a2Mes.add(0.158669, 0.147112);
        a2Mes.add(0.188306, 0.105152);
        a2Mes.add(0.216734, 0.154369);
        a2Mes.add(0.246371, 0.17321);
        a2Mes.add(0.272984, 0.283191);
        a2Mes.add(0.298992, 0.308039);
        a2Mes.add(0.317742, 0.3753);
        b1Mes.add(0.124194, 1.86709);
        b1Mes.add(0.158065, 1.85561);
        b1Mes.add(0.187702, 1.93525);
        b1Mes.add(0.217339, 1.9845);
        b1Mes.add(0.246976, 2.00942);
        b1Mes.add(0.272984, 2.04034);
        b1Mes.add(0.297782, 2.11381);
        b1Mes.add(0.318952, 2.1568);
        b2Mes.add(0.125403, -0.151488);
        b2Mes.add(0.158669, -0.175134);
        b2Mes.add(0.188306, -0.150213);
        b2Mes.add(0.217339, -0.192186);
        b2Mes.add(0.246976, -0.191585);
        b2Mes.add(0.273589, -0.172805);
        b2Mes.add(0.297782, -0.251356);
        b2Mes.add(0.316532, -0.275297);

        // Deformation series
        final XYSeries u0 = new XYSeries("u0");
        final XYSeries u1 = new XYSeries("u1");
        final XYSeries v1 = new XYSeries("v1");
        final XYSeries u2 = new XYSeries("u2");
        final XYSeries v2 = new XYSeries("v2");
        final XYSeries u0Mes = new XYSeries("u0Mes");
        final XYSeries u1Mes = new XYSeries("u1Mes");
        final XYSeries v1Mes = new XYSeries("v1Mes");
        final XYSeries u2Mes = new XYSeries("u2Mes");
        final XYSeries v2Mes = new XYSeries("v2Mes");
        u0Mes.add(0.124294, -1.12491);
        u0Mes.add(0.15801, -0.56805);
        u0Mes.add(0.187767, -1.16924);
        u0Mes.add(0.216408, -0.90444);
        u0Mes.add(0.245059, -1.37745);
        u0Mes.add(0.272485, -1.1186);
        u0Mes.add(0.316226, -1.63003);
        u1Mes.add(0.124698, 0.125037);
        u1Mes.add(0.157095, -0.202085);
        u1Mes.add(0.186916, 0.0259858);
        u1Mes.add(0.216432, -0.123955);
        u1Mes.add(0.245668, 0.128577);
        u1Mes.add(0.272711, -0.0881355);
        u1Mes.add(0.297237, 0.323508);
        u1Mes.add(0.316157, 0.534615);
        u2Mes.add(0.124599, 0.00309765);
        u2Mes.add(0.157341, 0.102763);
        u2Mes.add(0.18696, 0.0808585);
        u2Mes.add(0.216678, 0.180893);
        u2Mes.add(0.272151, 0.716811);
        u2Mes.add(0.296755, 0.476006);
        u2Mes.add(0.316929, 0.741838);
        v1Mes.add(0.124473, -0.515677);
        v1Mes.add(0.157073, -0.691047);
        v1Mes.add(0.186709, -0.774532);
        v1Mes.add(0.214996, -1.04685);
        v1Mes.add(0.245742, -1.27074);
        v1Mes.add(0.271057, -1.46336);
        v1Mes.add(0.29761, -1.61958);
        v1Mes.add(0.316028, -2.1283);
        v2Mes.add(0.123652, 0.0210252);
        v2Mes.add(0.156862, -0.148334);
        v2Mes.add(0.186556, -0.152559);
        v2Mes.add(0.215644, -0.156697);
        v2Mes.add(0.245966, -0.130524);
        v2Mes.add(0.272426, -0.414776);
        v2Mes.add(0.297922, -0.357428);
        v2Mes.add(0.317434, -0.19557);

        // Profile drag coefficient
        final XYSeries delta = new XYSeries("δ");

        // Rotor incidence 
        final XYSeries incidence = new XYSeries("incidence");

        // Rotor speed 
        final XYSeries rotorSpeed = new XYSeries("Rotor Speed");
        final XYSeries rotorSpeedMes = new XYSeries("Rotor SpeedMes");
        for (int i = 0; i < rotorSpeedTab.length; i++) {
            rotorSpeedMes.add(velocityTab[i] * 1.609344 / 3.6, // conversion to m/s
                    rotorSpeedTab[i]);
        }

        // Thrust coefficient 
        final XYSeries CT1 = new XYSeries("CT1");
        final XYSeries CT2 = new XYSeries("CT2");
        final XYSeries CTMes = new XYSeries("CTMes");
        CTMes.add(34.8749 * 1.609344 / 3.6, 0.00441338); // Speed converted to m/s
        CTMes.add(45.1119 * 1.609344 / 3.6, 0.00435093);
        CTMes.add(54.8978 * 1.609344 / 3.6, 0.00427509);
        CTMes.add(65.566 * 1.609344 / 3.6, 0.00417249);
        CTMes.add(74.4431 * 1.609344 / 3.6, 0.00405204);
        CTMes.add(83.9695 * 1.609344 / 3.6, 0.00387807);
        CTMes.add(93.258 * 1.609344 / 3.6, 0.00366394);
        CTMes.add(99.8847 * 1.609344 / 3.6, 0.00348996);

        // Inflow ratio
        final XYSeries lambda = new XYSeries("lambda");
        final XYSeries lambdaMes = new XYSeries("lambdaMes");
        lambdaMes.add(0.125, 0.0214);
        lambdaMes.add(0.150, 0.0213);
        lambdaMes.add(0.175, 0.0212);
        lambdaMes.add(0.200, 0.0209);
        lambdaMes.add(0.225, 0.0204);
        lambdaMes.add(0.250, 0.0197);
        lambdaMes.add(0.275, 0.0191);
        lambdaMes.add(0.300, 0.0185);
        lambdaMes.add(0.325, 0.0179);

        // Deformation along radius
        final XYSeries u0r = new XYSeries("u0");
        final XYSeries v1r = new XYSeries("v1");

        // Start point (incidence, profil drag coefficient)
        final double[] startPoint = {FastMath.toRadians(10), FastMath.toRadians(0.0143)};

        // Loop
        final double vs1 = velocityTab[0];
        final double vs2 = velocityTab[velocityTab.length - 1];
        final double nb = 30;
        final double dv = (vs2 - vs1) / (nb - 1);
        for (int i = 0; i < nb; i++) {
            double velocity = (vs1 + dv * i);
            final double Omega = rotorSpeedFunction.value(velocity) * FastMath.PI / 30; // Conversion to rad/s
            velocity = velocity * 1.609344 / 3.6; // Conversion to m/s;
            System.out.println("Velocity    : " + velocity);
            System.out.println("RotorSpeed  : " + Omega);
            PointValuePair result = kd1.compute(velocity, Omega, startPoint);
            final double mu = kd1.getRotor().getAdvanceRatio();

            // Flapping coefficient
            a0.add(mu, FastMath.toDegrees(kd1.getRotor().getConingAngle()));
            a1.add(mu, FastMath.toDegrees(kd1.getRotor().getLongitudinalFlapping()));
            b1.add(mu, FastMath.toDegrees(kd1.getRotor().getLateralFlapping()));
            a2.add(mu, FastMath.toDegrees(kd1.getRotor().get2ndLongitudinalFlapping()));
            b2.add(mu, FastMath.toDegrees(kd1.getRotor().get2ndLateralFlapping()));

            // Deformation coefficient at 0.75 R
            u0.add(mu, FastMath.toDegrees(kd1.getRotor().computeMeanPitchDeformation(0.75)));
            u1.add(mu, FastMath.toDegrees(kd1.getRotor().computePitchDeformation1C(0.75)));
            v1.add(mu, FastMath.toDegrees(kd1.getRotor().computePitchDeformation1S(0.75)));
            u2.add(mu, FastMath.toDegrees(kd1.getRotor().computePitchDeformation2C(0.75)));
            v2.add(mu, FastMath.toDegrees(kd1.getRotor().computePitchDeformation2S(0.75)));

            // Incidence and blade drag coefficient
            incidence.add(mu, FastMath.toDegrees(result.getPoint()[0]));
            delta.add(mu, result.getPoint()[1]);

            // Rotor speed and thrust coefficient (velocity m/s)
            rotorSpeed.add(velocity, Omega * 30 / FastMath.PI);
            final double ct = weight / (atmosphere.getDensity() * FastMath.pow(Omega, 2)
                    * FastMath.PI * FastMath.pow(R, 4));
            CT1.add(velocity, ct);
            CT2.add(velocity, kd1.getRotor().getThrust() / kd1.getRotor().getForceScaleFactor()
                    * kd1.getRotor().getSolidity());

            // Inflow coefficient
            final double lambdaD = kd1.getRotor().getInFlowRatio()
                    + mu * kd1.getRotor().getLongitudinalFlapping();
            lambda.add(mu, lambdaD);

            // Output
            System.out.println("eq1       : " + (kd1.getRotor().getLift() - weight));
            System.out.println("eq2       : " + kd1.getRotor().getTotalTorque());
            System.out.println("mu        : " + kd1.getRotor().getAdvanceRatio());
            System.out.println("lambda    : " + kd1.getRotor().getInFlowRatio());
            System.out.println("incidence : " + FastMath.toDegrees(result.getPoint()[0]));
            System.out.println("delta     : " + result.getPoint()[1]);
            System.out.println();
        }

        // Compute deformation at final mu along radius
        for (double r = 0; r < 1; r = r + 0.05) {
            u0r.add(r, FastMath.toDegrees(kd1.getRotor().computeMeanPitchDeformation(r)));
            v1r.add(r, FastMath.toDegrees(kd1.getRotor().computePitchDeformation1S(r)));
        }

        // Deformation along radius plot ---------------------------------------
        // u0r collection
        final XYSeriesCollection u0rCollection = new XYSeriesCollection();
        u0rCollection.addSeries(u0r);
        u0rCollection.addSeries(v1r);

        // u0r Plot
        final NumberAxis xuvrAxis = new NumberAxis("x");
        final NumberAxis yuvrAxis = new NumberAxis("");
        final XYItemRenderer uvrRenderer = new XYLineAndShapeRenderer(true, false);
        uvrRenderer.setBaseToolTipGenerator(new StandardXYToolTipGenerator());
        final XYPlot u0rPlot = new XYPlot(u0rCollection,
                xuvrAxis, yuvrAxis, uvrRenderer);
        u0rPlot.setOrientation(PlotOrientation.VERTICAL);

        // u0r chart
        final JFreeChart uvrChart = new JFreeChart("",
                JFreeChart.DEFAULT_TITLE_FONT, u0rPlot, true);
        ChartTheme uvrChartTheme = new StandardChartTheme("JFree");
        uvrChartTheme.apply(uvrChart);

        // Flapping plot chart -------------------------------------------------
        // Collections
        final XYSeriesCollection flappingCollection = new XYSeriesCollection();
        flappingCollection.addSeries(a0Mes);
        flappingCollection.addSeries(a0);
        flappingCollection.addSeries(a1Mes);
        flappingCollection.addSeries(a1);
        flappingCollection.addSeries(b1Mes);
        flappingCollection.addSeries(b1);
        flappingCollection.addSeries(a2Mes);
        flappingCollection.addSeries(a2);
        flappingCollection.addSeries(b2Mes);
        flappingCollection.addSeries(b2);

        // Plot
        final NumberAxis xFlappingAxis = new NumberAxis("mu");
        xFlappingAxis.setRangeWithMargins(0.12, 0.32);
        final NumberAxis yFlappingAxis = new NumberAxis("deg");
        yFlappingAxis.setAutoRangeIncludesZero(false);
        final XYPlot flappingPlot = new XYPlot(flappingCollection, xFlappingAxis,
                yFlappingAxis, null);
        flappingPlot.setOrientation(PlotOrientation.VERTICAL);

        // Renderer
        final XYLineAndShapeRenderer flappingRenderer = new XYLineAndShapeRenderer(true, false);
        flappingRenderer.setSeriesLinesVisible(0, false);
        flappingRenderer.setSeriesVisibleInLegend(0, false);
        flappingRenderer.setSeriesShapesVisible(0, true);
        flappingRenderer.setSeriesPaint(0, Color.red);
        flappingRenderer.setSeriesPaint(1, Color.red);
        flappingRenderer.setSeriesLinesVisible(2, false);
        flappingRenderer.setSeriesVisibleInLegend(2, false);
        flappingRenderer.setSeriesShapesVisible(2, true);
        flappingRenderer.setSeriesPaint(2, Color.green);
        flappingRenderer.setSeriesPaint(3, Color.green);
        flappingRenderer.setSeriesLinesVisible(4, false);
        flappingRenderer.setSeriesVisibleInLegend(4, false);
        flappingRenderer.setSeriesShapesVisible(4, true);
        flappingRenderer.setSeriesPaint(4, Color.blue);
        flappingRenderer.setSeriesPaint(5, Color.blue);
        flappingRenderer.setSeriesLinesVisible(6, false);
        flappingRenderer.setSeriesVisibleInLegend(6, false);
        flappingRenderer.setSeriesShapesVisible(6, true);
        flappingRenderer.setSeriesPaint(6, Color.yellow);
        flappingRenderer.setSeriesPaint(7, Color.yellow);
        flappingRenderer.setSeriesLinesVisible(8, false);
        flappingRenderer.setSeriesVisibleInLegend(8, false);
        flappingRenderer.setSeriesShapesVisible(8, true);
        flappingRenderer.setSeriesPaint(8, Color.magenta);
        flappingRenderer.setSeriesPaint(9, Color.magenta);

        // Chart
        final JFreeChart flappingChart = new JFreeChart("",
                JFreeChart.DEFAULT_TITLE_FONT, flappingPlot, true);
        ChartTheme flappingChartTheme = new StandardChartTheme("JFree");
        flappingChartTheme.apply(flappingChart);
        flappingPlot.setRenderer(flappingRenderer);

        // Deformation plot chart ----------------------------------------------
        // U Collections
        final XYSeriesCollection UCollection = new XYSeriesCollection();
        UCollection.addSeries(u0Mes);
        UCollection.addSeries(u0);
        UCollection.addSeries(u1Mes);
        UCollection.addSeries(u1);
        UCollection.addSeries(u2Mes);
        UCollection.addSeries(u2);

        // U Plot
        final NumberAxis yUAxis = new NumberAxis("u");
        yUAxis.setAutoRangeIncludesZero(false);
        final XYPlot UPlot = new XYPlot(UCollection, null, yUAxis, null);
        UPlot.setOrientation(PlotOrientation.VERTICAL);

        // U Renderer
        final XYLineAndShapeRenderer URenderer = new XYLineAndShapeRenderer(true, false);
        URenderer.setSeriesLinesVisible(0, false);
        URenderer.setSeriesVisibleInLegend(0, false);
        URenderer.setSeriesShapesVisible(0, true);
        URenderer.setSeriesPaint(0, Color.red);
        URenderer.setSeriesPaint(1, Color.red);
        URenderer.setSeriesLinesVisible(2, false);
        URenderer.setSeriesVisibleInLegend(2, false);
        URenderer.setSeriesShapesVisible(2, true);
        URenderer.setSeriesPaint(2, Color.green);
        URenderer.setSeriesPaint(3, Color.green);
        URenderer.setSeriesLinesVisible(4, false);
        URenderer.setSeriesVisibleInLegend(4, false);
        URenderer.setSeriesShapesVisible(4, true);
        URenderer.setSeriesPaint(4, Color.blue);
        URenderer.setSeriesPaint(5, Color.blue);

        // V Collections
        final XYSeriesCollection VCollection = new XYSeriesCollection();
        VCollection.addSeries(v1Mes);
        VCollection.addSeries(v1);
        VCollection.addSeries(v2Mes);
        VCollection.addSeries(v2);

        // V Plot
        final NumberAxis yVAxis = new NumberAxis("v");
        yVAxis.setAutoRangeIncludesZero(false);
        final XYPlot VPlot = new XYPlot(VCollection, null, yVAxis, null);
        VPlot.setOrientation(PlotOrientation.VERTICAL);

        // V Renderer
        final XYLineAndShapeRenderer VRenderer = new XYLineAndShapeRenderer(true, false);
        VRenderer.setSeriesLinesVisible(0, false);
        VRenderer.setSeriesVisibleInLegend(0, false);
        VRenderer.setSeriesShapesVisible(0, true);
        VRenderer.setSeriesPaint(0, Color.yellow);
        VRenderer.setSeriesPaint(1, Color.yellow);
        VRenderer.setSeriesLinesVisible(2, false);
        VRenderer.setSeriesVisibleInLegend(2, false);
        VRenderer.setSeriesShapesVisible(2, true);
        VRenderer.setSeriesPaint(2, Color.magenta);
        VRenderer.setSeriesPaint(3, Color.magenta);

        // Combined plot 
        final NumberAxis xUVAxis = new NumberAxis("mu");
        xUVAxis.setRangeWithMargins(0.12, 0.32);
        final CombinedDomainXYPlot combinePlotUV = new CombinedDomainXYPlot(xUVAxis);
        combinePlotUV.add(UPlot, 1);
        combinePlotUV.add(VPlot, 1);

        // Chart
        final JFreeChart deformationChart = new JFreeChart("",
                JFreeChart.DEFAULT_TITLE_FONT, combinePlotUV, true);
        ChartTheme deformationChartTheme = new StandardChartTheme("JFree");
        deformationChartTheme.apply(deformationChart);
        UPlot.setRenderer(URenderer);
        VPlot.setRenderer(VRenderer);

        // Incidence and blade drag plot chart ---------------------------------
        // Incidence collection
        final XYSeriesCollection incidenceCollection = new XYSeriesCollection();
        incidenceCollection.addSeries(incidence);

        // Incidence plot
        final NumberAxis yIncidenceAxis = new NumberAxis("deg");
        yIncidenceAxis.setAutoRangeIncludesZero(false);
        final XYItemRenderer incidenceRenderer = new XYLineAndShapeRenderer(true, false);
        incidenceRenderer.setBaseToolTipGenerator(new StandardXYToolTipGenerator());
        final XYPlot incidencePlot = new XYPlot(incidenceCollection, null,
                yIncidenceAxis, incidenceRenderer);

        // Drag collection
        final XYSeriesCollection dragCollection = new XYSeriesCollection();
        dragCollection.addSeries(delta);

        // Drag plot
        final NumberAxis yDragAxis = new NumberAxis("");
        yDragAxis.setAutoRangeIncludesZero(false);
        final XYItemRenderer dragRenderer = new XYLineAndShapeRenderer(true, false);
        dragRenderer.setBaseToolTipGenerator(new StandardXYToolTipGenerator());
        final XYPlot dragPlot = new XYPlot(dragCollection, null, yDragAxis,
                dragRenderer);

        // Combined plot 
        final NumberAxis xIncidenceAndDragAxis = new NumberAxis("mu");
        xIncidenceAndDragAxis.setAutoRangeIncludesZero(false);
        final CombinedDomainXYPlot combinePlotIncidenceAndDrag
                = new CombinedDomainXYPlot(xIncidenceAndDragAxis);
        combinePlotIncidenceAndDrag.add(incidencePlot, 1);
        combinePlotIncidenceAndDrag.add(dragPlot, 1);

        // Chart
        final JFreeChart incidenceAndDragChart = new JFreeChart("",
                JFreeChart.DEFAULT_TITLE_FONT, combinePlotIncidenceAndDrag, true);
        ChartTheme incidenceAndDraChartTheme = new StandardChartTheme("JFree");
        incidenceAndDraChartTheme.apply(incidenceAndDragChart);

        // Rotor speed an thrust plot chart ------------------------------------
        // Rotor speed collection
        final XYSeriesCollection rotorSpeedCollection = new XYSeriesCollection();
        rotorSpeedCollection.addSeries(rotorSpeedMes);
        rotorSpeedCollection.addSeries(rotorSpeed);

        // Rotor speed plot
        final NumberAxis yRotorSpeedAxis = new NumberAxis("rpm");
        yRotorSpeedAxis.setAutoRangeIncludesZero(false);
        final XYPlot rotorSpeedPlot = new XYPlot(rotorSpeedCollection, null,
                yRotorSpeedAxis, null);

        // Rotor speed Renderer
        final XYLineAndShapeRenderer rotorSpeedRenderer = new XYLineAndShapeRenderer(true, false);
        rotorSpeedRenderer.setSeriesLinesVisible(0, false);
        rotorSpeedRenderer.setSeriesVisibleInLegend(0, false);
        rotorSpeedRenderer.setSeriesShapesVisible(0, true);
        rotorSpeedRenderer.setSeriesPaint(0, Color.red);
        rotorSpeedRenderer.setSeriesPaint(1, Color.red);
        rotorSpeedRenderer.setBaseToolTipGenerator(new StandardXYToolTipGenerator());

        // CT collection
        final XYSeriesCollection CTCollection = new XYSeriesCollection();
        CTCollection.addSeries(CTMes);
        CTCollection.addSeries(CT1);
        CTCollection.addSeries(CT2);

        // CT plot
        final NumberAxis yCTAxis = new NumberAxis("");
        yCTAxis.setAutoRangeIncludesZero(false);
        final XYPlot CTPlot = new XYPlot(CTCollection, null,
                yCTAxis, null);

        // CT Renderer
        final XYLineAndShapeRenderer CTRenderer = new XYLineAndShapeRenderer(true, false);
        CTRenderer.setSeriesLinesVisible(0, false);
        CTRenderer.setSeriesVisibleInLegend(0, false);
        CTRenderer.setSeriesShapesVisible(0, true);
        CTRenderer.setSeriesPaint(0, Color.BLUE);
        CTRenderer.setSeriesPaint(1, Color.BLUE);
        CTRenderer.setSeriesPaint(2, Color.GREEN);
        CTRenderer.setBaseToolTipGenerator(new StandardXYToolTipGenerator());

        // Combined plot 
        final NumberAxis xSpeedAndCTAxis = new NumberAxis("Speed (m/s)");
        xSpeedAndCTAxis.setAutoRangeIncludesZero(false);
        final CombinedDomainXYPlot combinePlotSpeedAndCT
                = new CombinedDomainXYPlot(xSpeedAndCTAxis);
        combinePlotSpeedAndCT.add(rotorSpeedPlot, 1);
        combinePlotSpeedAndCT.add(CTPlot, 1);

        // Chart
        final JFreeChart rotorSpeedAndCTChart = new JFreeChart("",
                JFreeChart.DEFAULT_TITLE_FONT, combinePlotSpeedAndCT, true);
        ChartTheme rotorSpeedAndCTChartTheme = new StandardChartTheme("JFree");
        rotorSpeedAndCTChartTheme.apply(rotorSpeedAndCTChart);
        rotorSpeedPlot.setRenderer(rotorSpeedRenderer);
        CTPlot.setRenderer(CTRenderer);

        // Inflow coefficient chart --------------------------------------------
        // Collections
        final XYSeriesCollection lambdaCollection = new XYSeriesCollection();
        lambdaCollection.addSeries(lambdaMes);
        lambdaCollection.addSeries(lambda);

        // Plot
        final NumberAxis xLambdaAxis = new NumberAxis("mu");
        xLambdaAxis.setRangeWithMargins(0.12, 0.32);
        final NumberAxis yLambdaAxis = new NumberAxis("");
        yLambdaAxis.setAutoRangeIncludesZero(false);
        final XYPlot lambdaPlot = new XYPlot(lambdaCollection, xLambdaAxis,
                yLambdaAxis, null);
        lambdaPlot.setOrientation(PlotOrientation.VERTICAL);

        // Renderer
        final XYLineAndShapeRenderer lambdaRenderer = new XYLineAndShapeRenderer(true, false);
        lambdaRenderer.setSeriesLinesVisible(0, false);
        lambdaRenderer.setSeriesVisibleInLegend(0, false);
        lambdaRenderer.setSeriesShapesVisible(0, true);
        lambdaRenderer.setSeriesPaint(0, Color.red);
        lambdaRenderer.setSeriesPaint(0, Color.red);

        // Chart
        final JFreeChart lambdaChart = new JFreeChart("",
                JFreeChart.DEFAULT_TITLE_FONT, lambdaPlot, true);
        ChartTheme lambdaChartTheme = new StandardChartTheme("JFree");
        lambdaChartTheme.apply(lambdaChart);
        lambdaPlot.setRenderer(lambdaRenderer);
        lambdaChart.removeLegend();

        // Show chart ----------------------------------------------------------
        // flappingChart or deformationChart or incidenceAndDragChart or
        // rotorSpeedAndCTChart or lambdaChart or uvrChart
        ChartFrame frame = new ChartFrame("KD1 Results", deformationChart);
        frame.setMinimumSize(new Dimension(400, 700));
        frame.pack();
        frame.setVisible(true);
    }
}
