/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr\
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info\
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package gui.Rotor;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JPanel;
import main.rotor.Rotor;
import org.apache.commons.math3.util.FastMath;

/**
 *
 * @author J. Fourcade
 */
public class AoaDistributionChart extends JPanel {

    final private int size;
    final private int gap = 40;
    private Graphics2D g2D;
    private int scale;

    // Advance ratio
    private double mu;

    // Aoa colors
    private Color[] colors;

    // Aoa line distribution
    private Line[] aoaLines;

    /**
     *
     * @param size
     */
    public AoaDistributionChart(int size) {
        this.size = size;
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);

        // Transform to graphic 2D
        g2D = (Graphics2D) g;

        // Render
        g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2D.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);

        // Get size
        final int depth = 2 * FastMath.round(FastMath.min(getWidth(), getHeight()) / 2);

        // Scale and translation
        scale = depth / 2 - gap;
        AffineTransform at = new AffineTransform();
        at.translate(depth / 2, depth / 2);
        g2D.transform(at);

        // Draw ticks
        final int tickLength = gap / 3;
        g2D.drawLine(0, scale, 0, scale + tickLength); // 0deg
        g2D.drawLine(scale, 0, scale + tickLength, 0); // 90deg
        g2D.drawLine(0, -scale, 0, -scale - tickLength);  // 180deg
        g2D.drawLine(-scale, 0, -scale - tickLength, 0); // 270deg

        // Legends
        TextLayout t;
        String str;
        g2D.setColor(Color.BLACK);
        FontRenderContext frc = g2D.getFontRenderContext();
        Font font = new Font("Courier", Font.BOLD, 12);
        str = "0 deg";
        t = new TextLayout(str, font, frc);
        t.draw(g2D, -FastMath.round(t.getBounds().getWidth() / 2),
                scale + gap / 2 + FastMath.round(t.getBounds().getHeight()));
        str = "90";
        t = new TextLayout(str, font, frc);
        t.draw(g2D, scale + 5, -FastMath.round(t.getBounds().getHeight()));
        str = "180 deg";
        t = new TextLayout(str, font, frc);
        t.draw(g2D, -FastMath.round(t.getBounds().getWidth() / 2), -scale - gap / 2);
        str = "270";
        t = new TextLayout(str, font, frc);
        t.draw(g2D, -scale - 5 - FastMath.round(t.getBounds().getWidth()),
                -FastMath.round(t.getBounds().getHeight()));

        // Draw aoa distribution
        if (aoaLines != null) {
            for (int i = 0; i < aoaLines.length; i++) {
                g2D.setColor(colors[i]);
                g2D.setStroke(new BasicStroke(1.5F));
                aoaLines[i].draw();
            }
            // Inversion zone
            g2D.setColor(Color.darkGray);
            fillCircle(-(int) FastMath.round(scale * mu / 2), 0, mu / 2);
        }

        // Main circle
        g2D.setColor(Color.GRAY);
        g2D.setStroke(new BasicStroke(3F));
        drawCircle(0, 0, 1.0);
        fillCircle(0, 0, 0.10);
    }

    @Override
    public Dimension getPreferredSize() {
        return new Dimension(size, size);
    }

    /**
     *
     * @param xc
     * @param yc
     * @param radius
     */
    public void drawCircle(int xc, int yc, double radius) {
        final int r = (int) FastMath.round(scale * radius);
        g2D.drawOval(xc - r, yc - r, 2 * r, 2 * r);
    }

    /**
     *
     * @param xc
     * @param yc
     * @param radius
     */
    public void fillCircle(int xc, int yc, double radius) {
        final int r = (int) FastMath.round(scale * radius);
        g2D.fillOval(xc - r, yc - r, 2 * r, 2 * r);
    }

    /**
     *
     * @param rotor
     * @param aoa
     * @param colors
     */
    public void computeAoaDistribution(Rotor rotor, double[] aoa, Color[] colors) {
        mu = rotor.getAdvanceRatio();
        this.colors = colors;

        // Number of points in azimuth
        final int nbp = 1000;
        final double step = 2 * FastMath.PI / (nbp - 1);
        aoaLines = new Line[aoa.length];

        // Loop on azimuth angle
        for (int i = 0; i < aoa.length; i++) {
            g2D.setColor(colors[i]);
            aoaLines[i] = new Line();
            double azimuth = 0;
            double[] rp = new double[2];
            for (int j = 0; j < nbp; j++) {

                // Compute radial position (0,1 or 2 solution if twist)
                final double[] r = rotor.computeRadialPositionForGivenAoa(aoa[i], azimuth);
                for (int k = 0; k < 2; k++) {
                    if (r[k] != 0) {
                        aoaLines[i].addPoint(
                                (int) FastMath.round(scale * r[k] * FastMath.sin(azimuth)),
                                (int) FastMath.round(scale * r[k] * FastMath.cos(azimuth)),
                                k, rp[k] == 0);
                    }
                }
                rp[0] = r[0];
                rp[1] = r[1];
                azimuth = azimuth + step;
            }
        }
    }

    private class Line {

        final private List<Integer>[] x;
        final private List<Integer>[] y;
        final private List<Boolean>[] penOff;
        final private List<Integer>[] junction;

        public Line() {
            x = new List[2];
            y = new List[2];
            penOff = new List[2];
            for (int i = 0; i < 2; i++) {
                x[i] = new ArrayList<>();
                y[i] = new ArrayList<>();
                penOff[i] = new ArrayList<>();
            }
            junction = new List[2];
            junction[0] = new ArrayList<>();
            junction[1] = new ArrayList<>();
        }

        public void addPoint(int x, int y, int rank, boolean penOff) {
            this.x[rank].add(x);
            this.y[rank].add(y);
            this.penOff[rank].add(penOff);
        }

        public void draw() {
            List<Integer>[] xb = new List[2];
            List<Integer>[] yb = new List[2];
            for (int i = 0; i < 2; i++) {
                xb[i] = new ArrayList<>();
                yb[i] = new ArrayList<>();
            }
            // Draw lines
            for (int i = 0; i < 2; i++) {
                if (!x[i].isEmpty()) {
                    xb[i].add(x[i].get(0));
                    yb[i].add(y[i].get(0));
                    xb[i].add(x[i].get(x[i].size() - 1));
                    yb[i].add(y[i].get(x[i].size() - 1));
                }
                for (int j = 0; j < x[i].size() - 1; j++) {
                    if (!penOff[i].get(j + 1)) {
                        g2D.drawLine(x[i].get(j), y[i].get(j),
                                x[i].get(j + 1), y[i].get(j + 1));
                    } else {
                        xb[i].add(x[i].get(j));
                        yb[i].add(y[i].get(j));
                        xb[i].add(x[i].get(j + 1));
                        yb[i].add(y[i].get(j + 1));
                    }
                }
            }

            // Draw junctions (from one curve, only first and second index)
            List<Integer> removeIndex = new ArrayList<>();
            if (!x[0].isEmpty()) {
                if (drawJunction(xb[0].get(0), yb[0].get(0),
                        xb[0].get(1), yb[0].get(1))) {
                    removeIndex.add(0);
                    removeIndex.add(1);
                }
            }

            // Draw junctions (from one curve to the other)
            for (int i = 0; i < xb[0].size(); i++) {
                if (!removeIndex.contains(i)) {
                    for (int j = 0; j < xb[1].size(); j++) {
                        if (drawJunction(xb[0].get(i), yb[0].get(i),
                                xb[1].get(j), yb[1].get(j))) {
                            removeIndex.add(i);
                        }
                    }
                }
            }

            // Draw bounds (circle)
            for (int i = 0; i < xb[0].size(); i++) {
                if (!removeIndex.contains(i)) {
                    drawBound(xb[0].get(i), yb[0].get(i));
                }
            }
        }

        public boolean drawJunction(int x1, int y1, int x2, int y2) {
            final double norm = FastMath.sqrt(FastMath.pow(x2 - x1, 2)
                    + FastMath.pow(y2 - y1, 2)) / scale;
            if (norm < 0.15) {
                g2D.drawLine(x1, y1, x2, y2);
                return true;
            }
            return false;
        }

        private void drawBound(int x, int y) {
            final double norm = FastMath.sqrt(FastMath.pow(x, 2)
                    + FastMath.pow(y, 2)) / scale;
            if (norm > 0.6) {
                final double angle = FastMath.atan2(y, x);
                final int x1 = (int) FastMath.round(scale * FastMath.cos(angle));
                final int y1 = (int) FastMath.round(scale * FastMath.sin(angle));
                g2D.drawLine(x, y, x1, y1);
            }
        }
    }
}
