/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty and the software's author, the holder of the
 * economic rights, and the successive licensors  have only  limited
 * liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading, using, modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean that it is complicated to manipulate, and that also
 * therefore means that it is reserved for developers and experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and, more generally,to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package main.atmosphere;

import org.apache.commons.math3.util.FastMath;

/**
 * This class modelize the International Standard Atmophere US76 as defined by
 * the International Organization for Standardization.
 *
 * @author J. Fourcade
 */
public class StandardAtmosphere extends Atmosphere {

    // Constants
    final double kelvinToCelcius = 273.15;
    final double R = 287.053;
    final double gamma = 1.4;
    final private double g0 = 9.80665;
    final private double r0 = 6378128;
    final private double T0 = 288.15;
    final private double P0 = 101325;
    final private double kt = 0.0065;
    final private double kp = g0 / R / kt;
    // Datas
    final private double h, DT, T, P, g, phiH, rho, a, mu;

    /**
     * Constructor defining altitude and temperature increment.
     *
     * @param altitude : altide where atmosphere parameters are computed (m)
     * @param temperatureIncrement : temperature increment for non standard days (°C)
     */
    public StandardAtmosphere(double altitude, double temperatureIncrement) {
        h = altitude;
        DT = temperatureIncrement;

        // Temperature computation
        T = T0 - kt * h + DT;

        // Pression computation
        P = P0 * Math.pow(1 - kt * h / T0, kp);

        // Density
        rho = P / R / T;

        // Brake power altitude factor
        final double rho0 = P0 / R / T0;
        phiH = (rho / rho0 - 0.15) / 0.85;

        // Speed of sound
        a = Math.sqrt(gamma * R * T);

        // Acceleration of gravity computation
        g = g0 * Math.pow(r0 / (r0 + altitude), 2);

        // Viscosity
        mu = 0.1458e-5 * T * FastMath.sqrt(T) / (T + 110.4);
    }

    @Override
    public String getName() {
        return "International Standard Atmosphere";
    }

    /**
     *
     * @return altitude
     */
    public double getAltitude() {
        return h;
    }

    /**
     *
     * @return temperature increment
     */
    public double getTemperatureIncrement() {
        return DT;
    }

    @Override
    public double getTemperature() {
        return T;
    }

    @Override
    public double getPressure() {
        return P;
    }

    @Override
    public double getDensity() {
        return rho;
    }

    @Override
    public double getBrakePowerAltitudeFactor() {
        return phiH;
    }

    @Override
    public double getSpeedOfSound() {
        return a;
    }

    @Override
    public double getAccelerationOfGravity() {
        return g;
    }

    @Override
    public double getKinematicViscosity() {
        return mu;
    }
}
