/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty and the software's author, the holder of the
 * economic rights, and the successive licensors  have only  limited
 * liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading, using, modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean that it is complicated to manipulate, and that also
 * therefore means that it is reserved for developers and experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and, more generally,to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package main.inducedvelocity;

import main.exception.InducedVelocityException;
import main.rotor.TrustVelocitylRotor;
import org.apache.commons.math3.analysis.UnivariateFunction;
import org.apache.commons.math3.analysis.solvers.BrentSolver;

/**
 * Momentum induced velocity class implements constant induced velocity and is
 * based on momemtum theory.
 *
 * @author J. Fourcade
 */
public class MomentumInducedVelocity implements ConstantInducedVelocity {

    private double vi = 0;
    private TrustVelocitylRotor rotor;

    // Induced velocity function
    private class InducedVelocityZeroFunction implements UnivariateFunction {

        @Override
        public double value(double vi) {

            // Get vi0
            final double vi0 = rotor.computeTrustVelocity(vi);

            // Get barMu, barLambda, barV  ;
            final double barMu = rotor.getTangentialVelocity() / vi0;
            final double barLambda = -rotor.getNormalVelocity() / vi0;
            final double barV = vi / vi0;

            // Zero function
            return barV * Math.sqrt(
                    Math.pow(barMu, 2) + Math.pow(barLambda + barV, 2)) - 1;
        }
    }

    @Override
    public void compute(TrustVelocitylRotor rotor) {
        this.rotor = rotor;
        InducedVelocityZeroFunction f = new InducedVelocityZeroFunction();

        // Test zero induced velocity (no thrust)
        vi = 0;
        final double vi0 = rotor.computeTrustVelocity(0);
        if (vi0 != 0) {

            // Compute end value
            double endValue = -5;
            do {
                endValue = endValue + 0.1;
            } while (f.value(endValue) < 0 && endValue < 100);

            // Brent solver
            BrentSolver solver = new BrentSolver(1.e-12, 1.e-14);
            try {
                vi = solver.solve(100, f, -5, endValue);
            } catch (Exception ex) {
                throw new InducedVelocityException(
                        "Momentum induced velocity failed to converge :\n"
                        + "TangentialVelocity = "
                        + String.format("%.2f", rotor.getTangentialVelocity()) + " m/s"
                        + " NormalVelocity = "
                        + String.format("%.2f", rotor.getNormalVelocity()) + " m/s", ex);
            }
        }
    }

    @Override
    public double getVelocity() {
        return vi;
    }
}
