/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package main.rotor;

import main.atmosphere.Atmosphere;
import org.apache.commons.math3.geometry.euclidean.threed.Vector3D;
import org.apache.commons.math3.util.FastMath;

/**
 * Rotor interface.
 * 
 * @author J. Fourcade
 */
public abstract class Rotor {

    // Derivatives with respect to rotor non-feathering disk frame
    //  0-11 : L,D,T,Y,H,Q,a0,a1,b1,a2,b2,vi with respect to u,w,q,p,omega
    // in rotor frame
    final double[][] derivativesWRdisc = new double[12][5];

    // Derivatives with respect to fuselage frame
    // 0-5 : X,Y,Z,L,M,N,Q with respect to up,wp,q,pp,omega
    // in fuselage frame
    final double[][] derivativesWRfuselage = new double[7][5];

    /**
     * Compute state parameters of a given rotor in autorotative mode.
     * 
     * @param velocity : air speed of the rotor
     * @param incidence : incidence relative to non-feathering plane (rad)
     * @param bladePitch : blade pitch at root (rad)
     * @param pitchRate : pitch rate (rad/s)
     * @param rollRate : roll rate (rad/s)
     * @param atmosphere : atmophere used to get air density
     */
    public abstract void computeState(double velocity, double incidence, double bladePitch,
            double pitchRate, double rollRate, Atmosphere atmosphere);

    /**
     * Compute state parameters of a given rotor for a given rotor speed.
     * 
     * @param velocity : air speed of the rotor (m/s)
     * @param incidence : incidence relative to non-feathering plane (rad)
     * @param bladePitch : blade pitch at root (rad)
     * @param pitchRate : pitch rate (rad/s)
     * @param rollRate : roll rate (rad/s)
     * @param rotorSpeed : rotor speed (rad/s)
     * @param atmosphere : atmophere used to get air density
     */
    public abstract void computeState(double velocity, double incidence, double bladePitch,
            double pitchRate, double rollRate, double rotorSpeed, Atmosphere atmosphere);

    /**
     * Compute radial position along for a given angle of attack.
     * 
     * @param aoa : given aoa (deg)
     * @param azimuth : list of radial position where angle of attack is equal to aoa.
     * @return
     */
    public abstract double[] computeRadialPositionForGivenAoa(double aoa, double azimuth);

    /**
     * Compute rotor derivatives in two differents frames : non-feathering rotor
     * frame and fuselage frame. 
     *
     * @param velocity : air speed of the rotor (m/s)
     * @param rotorIncidence : incidence relative to non-feathering plane (rad)
     * @param fuselageIncidence : fuselage incidence
     * @param bladePitch : blade pitch at root (rad)
     * @param pitchRate : pitch rate (rad/s)
     * @param rollRate : roll rate (rad/s)
     * @param rotorSpeed : rotor speed (rad/s)
     * @param position : hub position in fuselage frame (X, Y, Z meters)
     * @param atmosphere : atmophere to compute air density
     */
    public void computeDerivatives(double velocity,
            double rotorIncidence, double bladePitch,
            double pitchRate, double rollRate, double rotorSpeed,
            double fuselageIncidence, Vector3D position, Atmosphere atmosphere) {

        // Mathematical variable
        final double U = velocity;
        final double alphaS = rotorIncidence;
        final double alphaF = fuselageIncidence;
        final double theta0 = bladePitch;
        final double q = pitchRate;
        final double p = rollRate;
        final double omega = rotorSpeed;
        final double u = U * FastMath.cos(alphaS);
        final double w = -U * FastMath.sin(alphaS);

        // Step value for derivative numerical computation
        final double minStep = 1e-4;
        final double factorStep = 1e-7;

        // Compute u and w and declare variables
        double Ud, alphaSd;
        double L, D, T, Y, H, Q, a0, a1, b1, a2, b2, vi;

        // u derivative --------------------------------------------------------
        // left
        double step = FastMath.max(u * factorStep, minStep);
        final double leftU = u - step;
        Ud = FastMath.sqrt(FastMath.pow(leftU, 2) + FastMath.pow(w, 2));
        alphaSd = FastMath.atan(-w / leftU);
        computeState(Ud, alphaSd, theta0, q, p, omega,
                atmosphere);
        L = getLift();
        D = getDrag();
        T = getThrust();
        Y = getLateralForce();
        H = getTotalRearForce();
        Q = getTotalTorque();
        a0 = getConingAngle();
        a1 = getLongitudinalFlapping();
        b1 = getLateralFlapping();
        a2 = get2ndLongitudinalFlapping();
        b2 = get2ndLateralFlapping();
        vi = getMeanInducedVelocity();
        // right
        final double rightU = u + step;
        Ud = FastMath.sqrt(FastMath.pow(rightU, 2) + FastMath.pow(w, 2));
        alphaSd = FastMath.atan(-w / rightU);
        computeState(Ud, alphaSd, theta0, q, p, omega,
                atmosphere);
        // derivatives
        derivativesWRdisc[0][0] = (getLift() - L) / (2 * step);
        derivativesWRdisc[1][0] = (getDrag() - D) / (2 * step);
        derivativesWRdisc[2][0] = (getThrust() - T) / (2 * step);
        derivativesWRdisc[3][0] = (getLateralForce() - Y) / (2 * step);
        derivativesWRdisc[4][0] = (getTotalRearForce() - H) / (2 * step);
        derivativesWRdisc[5][0] = (getTotalTorque() - Q) / (2 * step);
        derivativesWRdisc[6][0] = (getConingAngle() - a0) / (2 * step);
        derivativesWRdisc[7][0] = (getLongitudinalFlapping() - a1) / (2 * step);
        derivativesWRdisc[8][0] = (getLateralFlapping() - b1) / (2 * step);
        derivativesWRdisc[9][0] = (get2ndLongitudinalFlapping() - a2) / (2 * step);
        derivativesWRdisc[10][0] = (get2ndLateralFlapping() - b2) / (2 * step);
        derivativesWRdisc[11][0] = (getMeanInducedVelocity() - vi) / (2 * step);

        // w derivative --------------------------------------------------------
        // left
        step = FastMath.max(w * factorStep, minStep);
        final double leftW = w - step;
        Ud = FastMath.sqrt(FastMath.pow(u, 2) + FastMath.pow(leftW, 2));
        alphaSd = FastMath.atan(-leftW / u);
        computeState(Ud, alphaSd, theta0, q, p, omega,
                atmosphere);
        L = getLift();
        D = getDrag();
        T = getThrust();
        Y = getLateralForce();
        H = getTotalRearForce();
        Q = getTotalTorque();
        a0 = getConingAngle();
        a1 = getLongitudinalFlapping();
        b1 = getLateralFlapping();
        a2 = get2ndLongitudinalFlapping();
        b2 = get2ndLateralFlapping();
        vi = getMeanInducedVelocity();
        // right
        final double rightW = w + step;
        Ud = FastMath.sqrt(FastMath.pow(u, 2) + FastMath.pow(rightW, 2));
        alphaSd = FastMath.atan(-rightW / u);
        computeState(Ud, alphaSd, theta0, q, p, omega,
                atmosphere);
        // derivatives
        derivativesWRdisc[0][1] = (getLift() - L) / (2 * step);
        derivativesWRdisc[1][1] = (getDrag() - D) / (2 * step);
        derivativesWRdisc[2][1] = (getThrust() - T) / (2 * step);
        derivativesWRdisc[3][1] = (getLateralForce() - Y) / (2 * step);
        derivativesWRdisc[4][1] = (getTotalRearForce() - H) / (2 * step);
        derivativesWRdisc[5][1] = (getTotalTorque() - Q) / (2 * step);
        derivativesWRdisc[6][1] = (getConingAngle() - a0) / (2 * step);
        derivativesWRdisc[7][1] = (getLongitudinalFlapping() - a1) / (2 * step);
        derivativesWRdisc[8][1] = (getLateralFlapping() - b1) / (2 * step);
        derivativesWRdisc[9][1] = (get2ndLongitudinalFlapping() - a2) / (2 * step);
        derivativesWRdisc[10][1] = (get2ndLateralFlapping() - b2) / (2 * step);
        derivativesWRdisc[11][1] = (getMeanInducedVelocity() - vi) / (2 * step);

        // q derivative --------------------------------------------------------
        // left
        step = FastMath.max(q * factorStep, minStep);
        final double leftQ = q - step;
        computeState(U, alphaS, theta0, leftQ, p, omega,
                atmosphere);
        L = getLift();
        D = getDrag();
        T = getThrust();
        Y = getLateralForce();
        H = getTotalRearForce();
        Q = getTotalTorque();
        a0 = getConingAngle();
        a1 = getLongitudinalFlapping();
        b1 = getLateralFlapping();
        a2 = get2ndLongitudinalFlapping();
        b2 = get2ndLateralFlapping();
        vi = getMeanInducedVelocity();
        // right
        final double rightQ = q + step;
        computeState(U, alphaS, theta0, rightQ, p, omega,
                atmosphere);
        // derivatives
        derivativesWRdisc[0][2] = (getLift() - L) / (2 * step);
        derivativesWRdisc[1][2] = (getDrag() - D) / (2 * step);
        derivativesWRdisc[2][2] = (getThrust() - T) / (2 * step);
        derivativesWRdisc[3][2] = (getLateralForce() - Y) / (2 * step);
        derivativesWRdisc[4][2] = (getTotalRearForce() - H) / (2 * step);
        derivativesWRdisc[5][2] = (getTotalTorque() - Q) / (2 * step);
        derivativesWRdisc[6][2] = (getConingAngle() - a0) / (2 * step);
        derivativesWRdisc[7][2] = (getLongitudinalFlapping() - a1) / (2 * step);
        derivativesWRdisc[8][2] = (getLateralFlapping() - b1) / (2 * step);
        derivativesWRdisc[9][2] = (get2ndLongitudinalFlapping() - a2) / (2 * step);
        derivativesWRdisc[10][2] = (get2ndLateralFlapping() - b2) / (2 * step);
        derivativesWRdisc[11][2] = (getMeanInducedVelocity() - vi) / (2 * step);

        // p derivative --------------------------------------------------------
        // left
        step = FastMath.max(p * factorStep, minStep);
        final double leftP = p - step;
        computeState(U, alphaS, theta0, q, leftP, omega,
                atmosphere);
        L = getLift();
        D = getDrag();
        T = getThrust();
        Y = getLateralForce();
        H = getTotalRearForce();
        Q = getTotalTorque();
        a0 = getConingAngle();
        a1 = getLongitudinalFlapping();
        b1 = getLateralFlapping();
        a2 = get2ndLongitudinalFlapping();
        b2 = get2ndLateralFlapping();
        vi = getMeanInducedVelocity();
        // right
        final double rightP = p + step;
        computeState(U, alphaS, theta0, q, rightP, omega,
                atmosphere);
        // derivatives
        derivativesWRdisc[0][3] = (getLift() - L) / (2 * step);
        derivativesWRdisc[1][3] = (getDrag() - D) / (2 * step);
        derivativesWRdisc[2][3] = (getThrust() - T) / (2 * step);
        derivativesWRdisc[3][3] = (getLateralForce() - Y) / (2 * step);
        derivativesWRdisc[4][3] = (getTotalRearForce() - H) / (2 * step);
        derivativesWRdisc[5][3] = (getTotalTorque() - Q) / (2 * step);
        derivativesWRdisc[6][3] = (getConingAngle() - a0) / (2 * step);
        derivativesWRdisc[7][3] = (getLongitudinalFlapping() - a1) / (2 * step);
        derivativesWRdisc[8][3] = (getLateralFlapping() - b1) / (2 * step);
        derivativesWRdisc[9][3] = (get2ndLongitudinalFlapping() - a2) / (2 * step);
        derivativesWRdisc[10][3] = (get2ndLateralFlapping() - b2) / (2 * step);
        derivativesWRdisc[11][3] = (getMeanInducedVelocity() - vi) / (2 * step);

        // omega derivative ----------------------------------------------------
        // left
        step = FastMath.max(omega * factorStep, minStep);
        final double leftOmega = omega - step;
        computeState(U, alphaS, theta0, q, p, leftOmega,
                atmosphere);
        L = getLift();
        D = getDrag();
        T = getThrust();
        Y = getLateralForce();
        H = getTotalRearForce();
        Q = getTotalTorque();
        a0 = getConingAngle();
        a1 = getLongitudinalFlapping();
        b1 = getLateralFlapping();
        a2 = get2ndLongitudinalFlapping();
        b2 = get2ndLateralFlapping();
        vi = getMeanInducedVelocity();
        // right
        final double rightOmega = omega + step;
        computeState(U, alphaS, theta0, q, p, rightOmega,
                atmosphere);
        // derivatives
        derivativesWRdisc[0][4] = (getLift() - L) / (2 * step);
        derivativesWRdisc[1][4] = (getDrag() - D) / (2 * step);
        derivativesWRdisc[2][4] = (getThrust() - T) / (2 * step);
        derivativesWRdisc[3][4] = (getLateralForce() - Y) / (2 * step);
        derivativesWRdisc[4][4] = (getTotalRearForce() - H) / (2 * step);
        derivativesWRdisc[5][4] = (getTotalTorque() - Q) / (2 * step);
        derivativesWRdisc[6][4] = (getConingAngle() - a0) / (2 * step);
        derivativesWRdisc[7][4] = (getLongitudinalFlapping() - a1) / (2 * step);
        derivativesWRdisc[8][4] = (getLateralFlapping() - b1) / (2 * step);
        derivativesWRdisc[9][4] = (get2ndLongitudinalFlapping() - a2) / (2 * step);
        derivativesWRdisc[10][4] = (get2ndLateralFlapping() - b2) / (2 * step);
        derivativesWRdisc[11][4] = (getMeanInducedVelocity() - vi) / (2 * step);

        // Compute state with original parameters so that rotor state is correct
        computeState(U, alphaS, theta0, q, p, omega, atmosphere);
        L = getLift();
        D = getDrag();

        // Compute derivative of X and Z forces in fuselage frame with respect
        // to U coordinates in fuselage frame (TF)
        // Coordinate of U in TF : up, wp
        final double up = U * FastMath.cos(alphaF);
        final double wp = U * FastMath.sin(alphaF);

        // Derivative of L and D with respect to u and w (coordinates of U in
        // rotor frame (TS)
        final double DLDu = derivativesWRdisc[0][0];
        final double DLDw = derivativesWRdisc[0][1];
        final double DDDu = derivativesWRdisc[1][0];
        final double DDDw = derivativesWRdisc[1][1];

        // Frame transformation (TF to TS)
        final double Dalpha = alphaS - alphaF;
        final double cosDalpha = FastMath.cos(Dalpha);
        final double sinDalpha = FastMath.sin(Dalpha);

        // Derivative with respect to up and wp 
        final double DLDup = DLDu * cosDalpha - DLDw * sinDalpha;
        final double DLDwp = -DLDu * sinDalpha - DLDw * cosDalpha;
        final double DDDup = DDDu * cosDalpha - DDDw * sinDalpha;
        final double DDDwp = -DDDu * sinDalpha - DDDw * cosDalpha;

        // Compute X derivatives
        final double U3 = FastMath.pow(U, 3);
        derivativesWRfuselage[0][0] = up / U3 * (up * D - wp * L) + 1 / U * (-D - up * DDDup + wp * DLDup);
        derivativesWRfuselage[0][1] = wp / U3 * (up * D - wp * L) + 1 / U * (-up * DDDwp + L + wp * DLDwp);
        derivativesWRfuselage[0][2] = 1 / U * (-up * derivativesWRdisc[1][2] + wp * derivativesWRdisc[0][2]);
        derivativesWRfuselage[0][3] = 1 / U * (-up * derivativesWRdisc[1][3] + wp * derivativesWRdisc[0][3]);
        derivativesWRfuselage[0][4] = 1 / U * (-up * derivativesWRdisc[1][4] + wp * derivativesWRdisc[0][4]);

        // Compute Y derivatives
        derivativesWRfuselage[1][0] = derivativesWRdisc[3][0] * cosDalpha - derivativesWRdisc[3][1] * sinDalpha;
        derivativesWRfuselage[1][1] = -derivativesWRdisc[3][0] * sinDalpha - derivativesWRdisc[3][1] * cosDalpha;
        derivativesWRfuselage[1][2] = derivativesWRdisc[3][2];
        derivativesWRfuselage[1][3] = derivativesWRdisc[3][3];
        derivativesWRfuselage[1][4] = derivativesWRdisc[3][4];

        // Compute Z derivatives
        derivativesWRfuselage[2][0] = up / U3 * (wp * D + up * L) - 1 / U * (wp * DDDup + L + up * DLDup);
        derivativesWRfuselage[2][1] = wp / U3 * (wp * D + up * L) - 1 / U * (D + wp * DDDwp + up * DLDwp);
        derivativesWRfuselage[2][2] = 1 / U * (-wp * derivativesWRdisc[1][2] - up * derivativesWRdisc[0][2]);
        derivativesWRfuselage[2][3] = 1 / U * (-wp * derivativesWRdisc[1][3] - up * derivativesWRdisc[0][3]);
        derivativesWRfuselage[2][4] = 1 / U * (-wp * derivativesWRdisc[1][4] - up * derivativesWRdisc[0][4]);

        // Compute L-moment derivative
        derivativesWRfuselage[3][0] = position.getY() * derivativesWRfuselage[2][0] - position.getZ() * derivativesWRfuselage[1][0];
        derivativesWRfuselage[3][1] = position.getY() * derivativesWRfuselage[2][1] - position.getZ() * derivativesWRfuselage[1][1];
        derivativesWRfuselage[3][2] = position.getY() * derivativesWRfuselage[2][2] - position.getZ() * derivativesWRfuselage[1][2];
        derivativesWRfuselage[3][3] = position.getY() * derivativesWRfuselage[2][3] - position.getZ() * derivativesWRfuselage[1][3];
        derivativesWRfuselage[3][4] = position.getY() * derivativesWRfuselage[2][4] - position.getZ() * derivativesWRfuselage[1][4];

        // Compute M-moment derivative
        derivativesWRfuselage[4][0] = position.getZ() * derivativesWRfuselage[0][0] - position.getX() * derivativesWRfuselage[2][0];
        derivativesWRfuselage[4][1] = position.getZ() * derivativesWRfuselage[0][1] - position.getX() * derivativesWRfuselage[2][1];
        derivativesWRfuselage[4][2] = position.getZ() * derivativesWRfuselage[0][2] - position.getX() * derivativesWRfuselage[2][2];
        derivativesWRfuselage[4][3] = position.getZ() * derivativesWRfuselage[0][3] - position.getX() * derivativesWRfuselage[2][3];
        derivativesWRfuselage[4][4] = position.getZ() * derivativesWRfuselage[0][4] - position.getX() * derivativesWRfuselage[2][4];

        // Compute N-moment derivative
        derivativesWRfuselage[5][0] = position.getX() * derivativesWRfuselage[1][0] - position.getY() * derivativesWRfuselage[0][0];
        derivativesWRfuselage[5][1] = position.getX() * derivativesWRfuselage[1][1] - position.getY() * derivativesWRfuselage[0][1];
        derivativesWRfuselage[5][2] = position.getX() * derivativesWRfuselage[1][2] - position.getY() * derivativesWRfuselage[0][2];
        derivativesWRfuselage[5][3] = position.getX() * derivativesWRfuselage[1][3] - position.getY() * derivativesWRfuselage[0][3];
        derivativesWRfuselage[5][4] = position.getX() * derivativesWRfuselage[1][4] - position.getY() * derivativesWRfuselage[0][4];

        // Compute Q derivative
        derivativesWRfuselage[6][0] = derivativesWRdisc[5][0] * cosDalpha - derivativesWRdisc[5][1] * sinDalpha;
        derivativesWRfuselage[6][1] = -derivativesWRdisc[5][0] * sinDalpha - derivativesWRdisc[5][1] * cosDalpha;
        derivativesWRfuselage[6][2] = derivativesWRdisc[5][2];
        derivativesWRfuselage[6][3] = derivativesWRdisc[5][3];
        derivativesWRfuselage[6][4] = derivativesWRdisc[5][4];
    }

    /**
     * Get derivatives with respect to rotor non-feathering disk frame.
     * 
     * @return matrix[12][5] of derivatives. Order is : L,D,T,Y,H,Q,a0,a1,b1,a2,b2,vi 
     * with respect to u,w,q,p,omega

     */
    public double[][] getDerivativesWRdisc() {
        return derivativesWRdisc;
    }

    /**
     * Get derivatives with respect to fuselage frame.
     * 
     * @return matrix[5][5] of derivatives. Order is : X,Y,Z,L,M,N,Q 
     * with respect to u,w,q,p,omega
     */
    public double[][] getDerivativesWRfuselage() {
        return derivativesWRfuselage;
    }

    /**
     *
     * @return airspeed of the rotor (m/s)
     */
    public abstract double getVelocity();

    /**
     *
     * @return rotor speed (rad/s)
     */
    public abstract double getRotorSpeed();

    /**
     *
     * @return rotor incidence relative to no-feathering plane (deg)
     */
    public abstract double getIncidence();

    /**
     *
     * @return blade pitch at root (rad)
     */
    public abstract double getBladePitch();

    /**
     *
     * @return pitch rate (rad/s)
     */
    public abstract double getPitchRate();

    /**
     *
     * @return rool rate (rad/s)
     */
    public abstract double getRollRate();

    /**
     *
     * @return advance ratio μ
     */
    public abstract double getAdvanceRatio();

    /**
     *
     * @return inflow ratio λ
     */
    public abstract double getInFlowRatio();

    /**
     *
     * @return conning angle a0 (rad)
     */
    public abstract double getConingAngle();

    /**
     *
     * @return longitudinal flapping coefficient a1 (rad)
     */
    public abstract double getLongitudinalFlapping();

    /**
     *
     * @return lateral flapping coefficient b1 (rad)
     */
    public abstract double getLateralFlapping();

    /**
     *
     * @return 2nd oder longitudinal flapping coefficient a2 (rad)
     */
    public abstract double get2ndLongitudinalFlapping();

    /**
     *
     * @return 2nd order lateral flapping coefficient b2 (rad)
     */
    public abstract double get2ndLateralFlapping();

    /**
     *
     * @return rotor lift (N)
     */
    public abstract double getLift();

    /**
     *
     * @return rotor drag (N)
     */
    public abstract double getDrag();

    /**
     *
     * @return rotor thrust T (N)
     */
    public abstract double getThrust();

    /**
     *
     * @return lateral force Y (N)
     */
    public abstract double getLateralForce();

    /**
     *
     * @return profile drag rear force Hp (N)
     */
    public abstract double getProfileDragRearForce();

    /**
     *
     * @return induced rear force Hi (N)
     */
    public abstract double getInducedRearForce();

    /**
     *
     * @return total rear force Hp+Hi (N)
     */
    public abstract double getTotalRearForce();

    /**
     *
     * @return induced rotor torque Qi (N.m)
     */
    public abstract double getInducedTorque();

    /**
     *
     * @return profile drag rotor torque Qp (N.m)
     */
    public abstract double getProfileDragTorque();

    /**
     *
     * @return total rotor torque Qp+Qi (N.m)
     */
    public abstract double getTotalTorque();

    /**
     *
     * @return mean induced velocity (m/s)
     */
    public abstract double getMeanInducedVelocity();
}
