/*
 * GyroKit Copyright 2011-2013 J. Fourcade.
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty and the software's author, the holder of the
 * economic rights, and the successive licensors  have only  limited
 * liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading, using, modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean that it is complicated to manipulate, and that also
 * therefore means that it is reserved for developers and experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and, more generally,to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */package main.utils;

import org.apache.commons.math3.linear.Array2DRowRealMatrix;
import org.apache.commons.math3.linear.ArrayRealVector;
import org.apache.commons.math3.linear.DecompositionSolver;
import org.apache.commons.math3.linear.LUDecomposition;
import org.apache.commons.math3.linear.RealMatrix;
import org.apache.commons.math3.linear.RealVector;
import org.apache.commons.math3.util.FastMath;

/**
 * Compute polynomial coefficients to fit lookup table whith least square 
 * criterium
 * 
 * @author J. Fourcade
 */
public class PolynomialFitting {
    
    double[] coeff;

    /**
     * Constructor of the classe : gives order of the polynom and X (absciss) 
     * and Y (ordinate) tabulated values
     * 
     * @param order : polynom order
     * @param Y : ordinate table
     * @param X : absciss table
     */
    public PolynomialFitting(int order, double[] Y, double[] X) {

        // Least square matrix AX=B
        final double[][] m = new double[Y.length][order];
        for (int i = 0; i < Y.length; i++) {
            for (int j = 0; j < order; j++) {
               m[i][j] = FastMath.pow(X[i], j);
            }
        }
        RealMatrix coefficients = new Array2DRowRealMatrix(m);
        
        // Compute least square matrix ATA and ATB
        RealMatrix ATA = coefficients.preMultiply(coefficients.transpose());
        RealVector ATB = coefficients.transpose().operate(new ArrayRealVector(Y)) ;
        
        DecompositionSolver solver = new LUDecomposition(ATA).getSolver();
        RealVector solution = solver.solve(ATB);
        
        // Polynomial coefficients
        coeff = new double[order];
        for(int i=0;i<order;i++) {
            coeff[i]=solution.getEntry(i);
        }
    }
    
    /**
     * Compute value ordinate for a given absciss
     * @param x : absciss
     * @return ordinate
     */
    public double value(double x) {
        double y=0;
        for (int i=0;i<coeff.length;i++) {
            y=y+coeff[i]*FastMath.pow(x,i);
        }
        return y;
    }
}
