/*
 * GyroKit Copyright 2011-2013 J. Fourcade.
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package main.utils;

import java.util.Collections;
import java.util.EnumMap;
import java.util.Map;

/**
 * Units conversion class.
 *
 * @author J. Fourcade
 */
public class Units {

    /**
     * Unit mode.
     */
    static public enum Mode {

        /**
         * Units are Metric.
         */
        metric,
        /**
         * Units are British.
         */
        british
    };
    /**
     * Initial mode is Metric.
     */
    static public Mode mode = Mode.metric;

    /**
     *
     */
    static public enum UnitsList {

        /**
         * No units
         */
        none,
        /**
         * m of ft.
         */
        length1,
        /**
         * cm or inch.
         */
        length2,
        /**
         * kg or lb.
         */
        mass,
        /**
         * km/h or mph.
         */
        velocity1,
        /**
         * m/s ft/s.
         */
        velocity2,
        /**
         * m/s2 ot ft/s2.
         */
        acceleration,
        /**
         * daN or lbf.
         */
        force,
        /**
         * kg/m3 or slugs/ft3.
         */
        density,
        /**
         * kg/m or lb/ft.
         */
        massPerLength,
        /**
         * daN.m or lb.ft.
         */
        moment,
        /**
         * kg.m2 or lb.ft2.
         */
        momentOfInertia,
        /**
         * hp (europ) or hp (US).
         */
        power,
        /**
         * degres.
         */
        angle,
        /**
         * rpm.
         */
        rpm,
        /**
         * rpm/s.
         */
        rpmAcceleration,
        /**
         * °C.
         */
        temperature,
        /**
         * deg/s
         */
        angularVelocity,
        /**
         * N.m2/rad
         */
        torsionalRigidity,
        /**
         * rad/s2
         */
        radianAngularAcceleration,
        /**
         * rad/s
         */
        radianAngularVelocity
    };

    static private class OneUnit {

        String metricLabel;
        double metricConvertToSi;
        String britishLabel;
        double britishConvertToSi;

        public OneUnit(String metricLabel, double metricConvertToSi,
                String britishLabel, double britishConvertToSi) {
            this.metricLabel = metricLabel;
            this.metricConvertToSi = metricConvertToSi;
            this.britishLabel = britishLabel;
            this.britishConvertToSi = britishConvertToSi;
        }
    }
    final private static Map<UnitsList, OneUnit> unitsTable;

    static {
        Map<UnitsList, OneUnit> table = new EnumMap<>(UnitsList.class);
        table.put(UnitsList.length1,
                new OneUnit("m", 1, "ft", 0.3048));
        table.put(UnitsList.length2,
                new OneUnit("cm", 0.01, "inch", 0.0254));
        table.put(UnitsList.mass,
                new OneUnit("kg", 1, "lb", 0.45359237));
        table.put(UnitsList.velocity1,
                new OneUnit("km/h", 1 / 3.6, "mph", 0.44704));
        table.put(UnitsList.velocity2,
                new OneUnit("m/s", 1, "ft/s", 0.3048));
        table.put(UnitsList.acceleration,
                new OneUnit("m/s2", 1, "ft/s2", 0.3048));
        table.put(UnitsList.force,
                new OneUnit("daN", 10, "lbf", 4.448221600019));
        table.put(UnitsList.density,
                new OneUnit("kg/m3", 1, "slugs/ft3", 515.378818393));
        table.put(UnitsList.massPerLength,
                new OneUnit("kg/m", 1, "lb/ft", 1.48816394));
        table.put(UnitsList.moment,
                new OneUnit("daN.m", 10, "lbf.ft", 0.13825495));
        table.put(UnitsList.power,
                new OneUnit("hp", 735.5, "hp", 745.6998));
        table.put(UnitsList.momentOfInertia,
                new OneUnit("kg.m2", 1, "lb.ft2", 10.042140109));
        table.put(UnitsList.angle,
                new OneUnit("deg", Math.toRadians(1.0), "deg", Math.toRadians(1.0)));
        table.put(UnitsList.rpm,
                new OneUnit("rpm", Math.PI / 30, "rpm", Math.PI / 30));
        table.put(UnitsList.rpmAcceleration,
                new OneUnit("rpm/s", Math.PI / 30, "rpm/s", Math.PI / 30));
        table.put(UnitsList.temperature,
                new OneUnit("°C", 1, "°C", 1));
        table.put(UnitsList.angularVelocity,
                new OneUnit("deg/s", Math.toRadians(1.0), "deg/s", Math.toRadians(1.0)));
        table.put(UnitsList.torsionalRigidity,
                new OneUnit("N.m2/rad", 1, "lbf.inch2/rad",6894.75727));
        table.put(UnitsList.radianAngularAcceleration,
                new OneUnit("rad/s2", 1, "rad/s2",1));
         table.put(UnitsList.radianAngularVelocity,
                new OneUnit("rad/s", 1, "rad/s",1));
        unitsTable = Collections.unmodifiableMap(table);
    }

    /**
     * Convert value from unit to SI.
     *
     * @param value Value to be converted.
     * @param unit Unit.
     * @return Converted value.
     */
    static public double convertValueToSI(double value, UnitsList unit) {
        double factor = 1.0;
        if (unit != UnitsList.none) {
            switch (mode) {
                case metric:
                    factor = unitsTable.get(unit).metricConvertToSi;
                    break;
                case british:
                    factor = unitsTable.get(unit).britishConvertToSi;
                    break;
            }
        }
        return factor * value;
    }

    /**
     * Convert value from SI to unit.
     *
     * @param value Value to be converted.
     * @param unit Unit.
     * @return Converted value.
     */
    static public double convertValueFromSI(double value, UnitsList unit) {
        double factor = 1.0;
        if (unit != UnitsList.none) {
            switch (mode) {
                case metric:
                    factor = unitsTable.get(unit).metricConvertToSi;
                    break;
                case british:
                    factor = unitsTable.get(unit).britishConvertToSi;
                    break;
            }
        }
        return value / factor;
    }

    /**
     * Convert derivative value from Unit to SI.
     *
     * @param value Value to be converted.
     * @param numeratorUnit Numerator unit.
     * @param denominatorUnit Denominator unit
     * @return Converted value.
     */
    static public double convertDerivativeToSI(double value, UnitsList numeratorUnit,
            UnitsList denominatorUnit) {
        double factor = 0.0;
        switch (mode) {
            case metric:
                factor = unitsTable.get(numeratorUnit).metricConvertToSi;
                factor = factor / unitsTable.get(denominatorUnit).metricConvertToSi;
                break;
            case british:
                factor = unitsTable.get(numeratorUnit).britishConvertToSi;
                factor = factor / unitsTable.get(denominatorUnit).britishConvertToSi;
                break;
        }
        return factor * value;
    }

    /**
     * Convert derivative value from SI to Unit.
     *
     * @param value Value to be converted.
     * @param numeratorUnit Numerator unit
     * @param denominatorUnit Denominator unit
     * @return Converted value.
     */
    static public double convertDerivativeFromSI(double value, UnitsList numeratorUnit,
            UnitsList denominatorUnit) {
        double factor = 0.0;
        switch (mode) {
            case metric:
                factor = unitsTable.get(numeratorUnit).metricConvertToSi;
                factor = factor / unitsTable.get(denominatorUnit).metricConvertToSi;
                break;
            case british:
                factor = unitsTable.get(numeratorUnit).britishConvertToSi;
                factor = factor / unitsTable.get(denominatorUnit).britishConvertToSi;
                break;
        }
        return value / factor;
    }

    /**
     * Convert from one system unit (British or metric) to the other.
     *
     * @param value Value to be converted
     * @param unit Unit of one of British or metric input.
     * @return Converted value.
     */
    static public double unitSystemConvert(double value, UnitsList unit) {
        double factor = unitsTable.get(unit).metricConvertToSi
                / unitsTable.get(unit).britishConvertToSi;
        switch (mode) {
            case metric:
                return value / factor;
            case british:
                return value * factor;
        }
        return value;
    }

    /**
     * Get label of a given unit.
     *
     * @param unit Unit name.
     * @return Unit label.
     */
    static public String getLabel(UnitsList unit) {
        String unitLabel = "";
        if (unit != UnitsList.none) {
            switch (mode) {
                case metric:
                    unitLabel = unitsTable.get(unit).metricLabel;
                    break;
                case british:
                    unitLabel = unitsTable.get(unit).britishLabel;
                    break;
            }
        }
        return unitLabel;
    }

    /**
     * Get current mode (Metric or British).
     *
     * @return
     */
    static public String getMode() {
        if (mode == Mode.metric) {
            return "Metric";
        } else {
            return "British";
        }
    }
}
