/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package rotor;

import static junit.framework.Assert.assertEquals;
import main.airfoil.ConstantAirfoil;
import main.atmosphere.Atmosphere;
import main.atmosphere.DensityAtmosphere;
import main.inducedvelocity.ConstantInducedVelocity;
import main.inducedvelocity.ShaydakovInducedVelocity;
import main.rotor.GlauertRotor;
import org.apache.commons.math3.geometry.euclidean.threed.Vector3D;
import org.apache.commons.math3.util.FastMath;

/**
 * Glauert derivative rotor test
 *
 * @author J. Fourcade
 */
public class GlauertRotorDerivativeTest {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {

        // Define rotor
        final double a = 5.7;
        final double delta = 0.011;
        final double cm = 0;
        final double xac = 0;
        ConstantAirfoil bladeProfile = new ConstantAirfoil(a, delta, cm, xac);
        ConstantInducedVelocity inducedVelocity = new ShaydakovInducedVelocity();
        Atmosphere atmosphere = new DensityAtmosphere(1.225);
        final int b = 2;
        final double R = 4;
        final double c = 0.2;
        final double xcg = 0;
        final double thetaTW = FastMath.toRadians(4);
        final double GJ = 0;
        final double B = 0.97;
        final double Ib = 64.00;
        final double Ic = 64.00;
        final Vector3D position = new Vector3D(-0.3, -0.1, -1.2);
        GlauertRotor rotor = new GlauertRotor(b, R, c, xcg, thetaTW, GJ, B, Ib, Ic,
                bladeProfile, inducedVelocity);

        // Simulate in autorotative mode : 80 km/h, 10 degres incidence, 
        // 2 degres blade pitch, 2 deg/s pitch and -3 deg/s roll rate
        // fuselage incidence 3 deg
        final double U = 80 / 3.6;
        final double alphaS = Math.toRadians(10);
        final double alphaF = Math.toRadians(3);
        final double theta0 = Math.toRadians(2);
        final double q = FastMath.toRadians(2);
        final double p = FastMath.toRadians(-3);
        rotor.computeState(U, alphaS, theta0, q, p, atmosphere);

        // Get scale factors
        final double KF = rotor.getForceScaleFactor();
        final double KQ = rotor.getMomentScaleFactor();
        final double Kv = rotor.getVelocityScaleFactor();
        final double Kav = rotor.getAngularVelocityScaleFactor();

        // Get rotor values
        final double mu = rotor.getAdvanceRatio();
        final double lambda = rotor.getInFlowRatio();
        final double a0 = rotor.getConingAngle();
        final double a1 = rotor.getLongitudinalFlapping();
        final double b1 = rotor.getLateralFlapping();
        final double a2 = rotor.get2ndLongitudinalFlapping();
        final double b2 = rotor.get2ndLateralFlapping();
        final double gamma = rotor.getLockNumber();
        final double omega = rotor.getRotorSpeed();
        final double vi = rotor.getInducedVelocity().getVelocity();
        final double u = U * FastMath.cos(alphaS);
        final double w = -U * FastMath.sin(alphaS);
        final double CT = rotor.getThrust() / KF;
        final double CY = (rotor.getLateralForce()) / KF;
        final double CH = (rotor.getTotalRearForce()) / KF;
        final double CQ = (rotor.getTotalTorque()) / KQ;
        final double CL = rotor.getLift() / KF;
        final double CD = rotor.getDrag() / KF;

        // Compute derivatives
        rotor.computeState(U, alphaS, theta0,
                q, p, omega, atmosphere);

        // Print rotor speed
        System.out.println();
        System.out.println("Rotor speed (rpm) : " + rotor.getRotorSpeed() * 30 / FastMath.PI);

        // Compute derivatives
        rotor.computeDerivatives(U, alphaS, theta0,
                q, p, omega, alphaF, position, atmosphere);

        double[][] derivativesWRTdisc = rotor.getDerivativesWRdisc();
        double[][] derivativesWRTfuselage = rotor.getDerivativesWRfuselage();

        // Normalize pitch and roll rate
        final double qc = q / omega;
        final double pc = p / omega;

        // Compute power of B, mu and gamma
        final double B2 = B * B;
        final double B3 = B * B2;
        final double B4 = B * B3;
        final double B5 = B * B4;
        final double B6 = B * B5;
        final double B7 = B * B6;
        final double B8 = B * B7;
        final double B9 = B * B8;
        final double B10 = B * B9;
        final double B11 = B * B10;
        final double mu2 = mu * mu;
        final double mu3 = mu * mu2;
        final double mu4 = mu * mu3;
        final double gamma2 = gamma * gamma;

        // Normalise induced velocity
        final double lambdai = vi / Kv;

        // Normalise derivatives
        final double DCLDuN = derivativesWRTdisc[0][0] * Kv / KF;
        final double DCDDuN = derivativesWRTdisc[1][0] * Kv / KF;
        final double DCTDuN = derivativesWRTdisc[2][0] * Kv / KF;
        final double DCYDuN = derivativesWRTdisc[3][0] * Kv / KF;
        final double DCHDuN = derivativesWRTdisc[4][0] * Kv / KF;
        final double DCQDuN = derivativesWRTdisc[5][0] * Kv / KQ;
        final double Da0DuN = derivativesWRTdisc[6][0] * Kv;
        final double Da1DuN = derivativesWRTdisc[7][0] * Kv;
        final double Db1DuN = derivativesWRTdisc[8][0] * Kv;
        final double Da2DuN = derivativesWRTdisc[9][0] * Kv;
        final double Db2DuN = derivativesWRTdisc[10][0] * Kv;
        final double DlambdaiDuN = derivativesWRTdisc[11][0];

        final double DCLDwN = derivativesWRTdisc[0][1] * Kv / KF;
        final double DCDDwN = derivativesWRTdisc[1][1] * Kv / KF;
        final double DCTDwN = derivativesWRTdisc[2][1] * Kv / KF;
        final double DCYDwN = derivativesWRTdisc[3][1] * Kv / KF;
        final double DCHDwN = derivativesWRTdisc[4][1] * Kv / KF;
        final double DCQDwN = derivativesWRTdisc[5][1] * Kv / KQ;
        final double Da0DwN = derivativesWRTdisc[6][1] * Kv;
        final double Da1DwN = derivativesWRTdisc[7][1] * Kv;
        final double Db1DwN = derivativesWRTdisc[8][1] * Kv;
        final double Da2DwN = derivativesWRTdisc[9][1] * Kv;
        final double Db2DwN = derivativesWRTdisc[10][1] * Kv;
        final double DlambdaiDwN = derivativesWRTdisc[11][1];

        final double DCLDqN = derivativesWRTdisc[0][2] * Kav / KF;
        final double DCDDqN = derivativesWRTdisc[1][2] * Kav / KF;
        final double DCTDqN = derivativesWRTdisc[2][2] * Kav / KF;
        final double DCYDqN = derivativesWRTdisc[3][2] * Kav / KF;
        final double DCHDqN = derivativesWRTdisc[4][2] * Kav / KF;
        final double DCQDqN = derivativesWRTdisc[5][2] * Kav / KQ;
        final double Da0DqN = derivativesWRTdisc[6][2] * Kav;
        final double Da1DqN = derivativesWRTdisc[7][2] * Kav;
        final double Db1DqN = derivativesWRTdisc[8][2] * Kav;
        final double Da2DqN = derivativesWRTdisc[9][2] * Kav;
        final double Db2DqN = derivativesWRTdisc[10][2] * Kav;
        final double DlambdaiDqN = derivativesWRTdisc[11][2] * Kav / Kv;

        final double DCLDpN = derivativesWRTdisc[0][3] * Kav / KF;
        final double DCDDpN = derivativesWRTdisc[1][3] * Kav / KF;
        final double DCTDpN = derivativesWRTdisc[2][3] * Kav / KF;
        final double DCYDpN = derivativesWRTdisc[3][3] * Kav / KF;
        final double DCHDpN = derivativesWRTdisc[4][3] * Kav / KF;
        final double DCQDpN = derivativesWRTdisc[5][3] * Kav / KQ;
        final double Da0DpN = derivativesWRTdisc[6][3] * Kav;
        final double Da1DpN = derivativesWRTdisc[7][3] * Kav;
        final double Db1DpN = derivativesWRTdisc[8][3] * Kav;
        final double Da2DpN = derivativesWRTdisc[9][3] * Kav;
        final double Db2DpN = derivativesWRTdisc[10][3] * Kav;
        final double DlambdaiDpN = derivativesWRTdisc[11][3] * Kav / Kv;

        final double DCLDomegaN = derivativesWRTdisc[0][4] * Kav / KF - 2 * CL;
        final double DCDDomegaN = derivativesWRTdisc[1][4] * Kav / KF - 2 * CD;
        final double DCTDomegaN = derivativesWRTdisc[2][4] * Kav / KF - 2 * CT;
        final double DCYDomegaN = derivativesWRTdisc[3][4] * Kav / KF - 2 * CY;
        final double DCHDomegaN = derivativesWRTdisc[4][4] * Kav / KF - 2 * CH;
        final double DCQDomegaN = derivativesWRTdisc[5][4] * Kav / KQ - 2 * CQ;
        final double Da0DomegaN = derivativesWRTdisc[6][4] * Kav;
        final double Da1DomegaN = derivativesWRTdisc[7][4] * Kav;
        final double Db1DomegaN = derivativesWRTdisc[8][4] * Kav;
        final double Da2DomegaN = derivativesWRTdisc[9][4] * Kav;
        final double Db2DomegaN = derivativesWRTdisc[10][4] * Kav;
        final double DlambdaiDomegaN = derivativesWRTdisc[11][4] * Kav / Kv;

        // Compute derivatives of incidence
        final double DalphaSDuA = Kv * w / FastMath.pow(U, 2);
        final double DalphaSDwA = -Kv * u / FastMath.pow(U, 2);

        // Compute derivatives of mu
        final double DmuDomegaA = -mu;

        // Compute derivative of lambda
        final double DlambdaDuA = -DlambdaiDuN;
        final double DlambdaDwA = -1 - DlambdaiDwN;
        final double DlambdaDqA = -DlambdaiDqN;
        final double DlambdaDpA = -DlambdaiDpN;
        final double DlambdaDomegaA = -lambda - DlambdaiDomegaN;

        // Tol for test
        final double tolA = 1e-10; // angles and coefficient
        final double tolF = 1e-5; // forces

        // a2 derivatives ------------------------------------------------------
        // a2 value
        System.out.println();
        final double a2bis = gamma * mu2 / (gamma2 * B8 + 144)
                * ((7. / 144. * gamma2 * B10 + 46. / 3. * B2) * theta0
                + (7. / 180. * B11 * gamma2 + 12 * B3) * thetaTW
                + (16 * B + 7. / 108. * B9 * gamma2) * lambda
                - 20. / 3. * B3 * pc / mu
                - (128 / B / gamma + 1. / 3. * B7 * gamma) * qc / mu);
        System.out.println("a2 " + a2bis);
        System.out.println("a2 " + a2);
        // derivative with respect to u
        final double Da2DuA = 2 * a2 / mu + gamma * mu2 / (gamma2 * B8 + 144)
                * ((16 * B + 7. / 108. * B9 * gamma2) * DlambdaDuA
                + 20. / 3. * B3 * pc / mu2
                + (128 / B / gamma + 1. / 3. * B7 * gamma) * qc / mu2);
        System.out.println("Da2DuA " + Da2DuA);
        System.out.println("Da2DuN " + Da2DuN);
        assertEquals("Da2Du", Da2DuA, Da2DuN, tolA);
        // derivative with respect to w
        final double Da2DwA = gamma * mu2 / (gamma2 * B8 + 144)
                * ((16 * B + 7. / 108. * B9 * gamma2) * DlambdaDwA);
        System.out.println("Da2DwA " + Da2DwA);
        System.out.println("Da2DwN " + Da2DwN);
        assertEquals("Da2Dw", Da2DwA, Da2DwN, tolA);
        // derivative with respect to q
        final double Da2DqA = gamma * mu2 / (gamma2 * B8 + 144)
                * ((16 * B + 7. / 108. * B9 * gamma2) * DlambdaDqA
                - (128 / B / gamma + 1. / 3. * B7 * gamma) / mu);
        System.out.println("Da2DqA " + Da2DqA);
        System.out.println("Da2DqN " + Da2DqN);
        assertEquals("Da2Dq", Da2DqA, Da2DqN, tolA);
        // derivative with respect to p
        final double Da2DpA = gamma * mu2 / (gamma2 * B8 + 144)
                * ((16 * B + 7. / 108. * B9 * gamma2) * DlambdaDpA
                - 20. / 3. * B3 / mu);
        System.out.println("Da2DpA " + Da2DpA);
        System.out.println("Da2DpN " + Da2DpN);
        assertEquals("Da2Dp", Da2DpA, Da2DpN, tolA);
        // derivative with respect to omega
        final double Da2DomegaA = 2 * a2 / mu * DmuDomegaA
                + gamma * mu2 / (gamma2 * B8 + 144)
                * ((16 * B + 7. / 108. * B9 * gamma2) * DlambdaDomegaA
                + (20. / 3. * B3 * pc) * (1 / mu + 1 / mu2 * DmuDomegaA)
                + (128 / B / gamma + 1. / 3. * B7 * gamma)
                * (1 / mu + 1 / mu2 * DmuDomegaA));
        System.out.println("Da2DomegaA " + Da2DomegaA);
        System.out.println("Da2DomegaN " + Da2DomegaN);
        assertEquals("Da2Domega", Da2DomegaA, Da2DomegaN, tolA);

        // b2 derivatives ------------------------------------------------------
        // b2 value
        System.out.println();
        final double b2bis = -gamma2 * mu2 / (gamma2 * B8 + 144)
                * (25. / 36. * B6 * theta0 + 8. / 15. * B7 * thetaTW
                + 5. / 9. * B5 * lambda
                + (128. / (gamma2 * B) + 1. / 3. * B7) * pc / mu
                - 20. / 3. * B3 / gamma * qc / mu);
        System.out.println("b2 " + b2bis);
        System.out.println("b2 " + b2);
        // derivative with respect to u
        final double Db2DuA = 2 * b2 / mu - gamma2 * mu2 / (gamma2 * B8 + 144)
                * (5. / 9. * B5 * DlambdaDuA
                - (128. / (gamma2 * B) + 1. / 3. * B7) * pc / mu2
                + 20. / 3. * B3 / gamma * qc / mu2);
        System.out.println("Db2DuA " + Db2DuA);
        System.out.println("Db2DuN " + Db2DuN);
        assertEquals("Db2Du", Db2DuA, Db2DuN, tolA);
        // derivative with respect to w
        final double Db2DwA = -gamma2 * mu2 / (gamma2 * B8 + 144)
                * 5. / 9. * B5 * DlambdaDwA;
        System.out.println("Db2DwA " + Db2DwA);
        System.out.println("Db2DwN " + Db2DwN);
        assertEquals("Db2Dw", Db2DwA, Db2DwN, tolA);
        // derivative with respect to q
        final double Db2DqA = -gamma2 * mu2 / (gamma2 * B8 + 144)
                * (5. / 9. * B5 * DlambdaDqA
                - 20. / 3. * B3 / gamma / mu);
        System.out.println("Db2DqA " + Db2DqA);
        System.out.println("Db2DqN " + Db2DqN);
        assertEquals("Db2Dq", Db2DqA, Db2DqN, tolA);
        // derivative with respect to p
        final double Db2DpA = -gamma2 * mu2 / (gamma2 * B8 + 144)
                * (5. / 9. * B5 * DlambdaDpA
                + (128. / (gamma2 * B) + 1. / 3. * B7) / mu);
        System.out.println("Db2DpA " + Db2DpA);
        System.out.println("Db2DpN " + Db2DpN);
        assertEquals("Db2Dp", Db2DpA, Db2DpN, tolA);
        // derivative with respect to omega
        final double Db2DomegaA = 2 * b2 / mu * DmuDomegaA
                - gamma2 * mu2 / (gamma2 * B8 + 144)
                * (5. / 9. * B5 * DlambdaDomegaA
                - (128. / (gamma2 * B) + 1. / 3. * B7) * pc
                * (1 / mu + 1 / mu2 * DmuDomegaA)
                + 20. / 3. * B3 / gamma * qc * (1 / mu + 1 / mu2 * DmuDomegaA));
        System.out.println("Db2DomegaA " + Db2DomegaA);
        System.out.println("Db2DomegaN " + Db2DomegaN);
        assertEquals("Db2Domega", Db2DomegaA, Db2DomegaN, tolA);

        // a0 derivatives ------------------------------------------------------
        // a0 value
        System.out.println();
        final double a0bis = gamma / 2
                * ((1. / 4. * B4 - 1. / 32. * mu4 + 1. / 4. * mu2 * B2) * theta0
                + (1. / 5. * B5 + 1. / 6. * B3 * mu2) * thetaTW
                + (1. / 3. * B3 + 1. / 4. * mu3 / FastMath.PI) * lambda
                + 1. / 8. * mu2 * B2 * b2
                + (1. / 6. * B3 * mu - 5. / 48. * mu4 / FastMath.PI) * pc);
        System.out.println("a0 " + a0bis);
        System.out.println("a0 " + a0);
        // derivative with respect to u
        final double Da0DuA = gamma / 2
                * ((-1. / 8. * mu3 + 1. / 2. * B2 * mu) * theta0
                + 1. / 3. * B3 * mu * thetaTW
                + 3. / 4. * mu2 / FastMath.PI * lambda
                + (1. / 3. * B3 + 1. / 4. * mu3 / FastMath.PI) * DlambdaDuA
                + 1. / 4. * B2 * mu * b2
                + 1. / 8. * mu2 * B2 * Db2DuA
                + (1. / 6. * B3 - 5. / 12. * mu3 / FastMath.PI) * pc);
        System.out.println("Da0DuA " + Da0DuA);
        System.out.println("Da0DuN " + Da0DuN);
        assertEquals("Da0Du", Da0DuA, Da0DuN, tolA);
        // derivative with respect to w
        final double Da0DwA = gamma / 2
                * ((1. / 3. * B3 + 1. / 4. * mu3 / FastMath.PI) * DlambdaDwA
                + 1. / 8. * mu2 * B2 * Db2DwA);
        System.out.println("Da0DwA " + Da0DwA);
        System.out.println("Da0DwN " + Da0DwN);
        assertEquals("Da0Dw", Da0DwA, Da0DwN, tolA);
        // derivative with respect to q
        final double Da0DqA = gamma / 2
                * ((1. / 3. * B3 + 1. / 4. * mu3 / FastMath.PI) * DlambdaDqA
                + 1. / 8. * mu2 * B2 * Db2DqA);
        System.out.println("Da0DqA " + Da0DqA);
        System.out.println("Da0DqN " + Da0DqN);
        assertEquals("Da0Dq", Da0DqA, Da0DqN, tolA);
        // derivative with respect to p
        final double Da0DpA = gamma / 2
                * ((1. / 3. * B3 + 1. / 4. * mu3 / FastMath.PI) * DlambdaDpA
                + 1. / 8. * mu2 * B2 * Db2DpA
                + (1. / 6. * B3 * mu - 5. / 48. * mu4 / FastMath.PI));
        System.out.println("Da0DpA " + Da0DpA);
        System.out.println("Da0DpN " + Da0DpN);
        assertEquals("Da0Dp", Da0DpA, Da0DpN, tolA);
        // derivative with respect to omega
        final double Da0DomegaA = gamma / 2
                * ((-1. / 8. * mu3 + 1. / 2. * B2 * mu) * theta0 * DmuDomegaA
                + 1. / 3. * B3 * mu * thetaTW * DmuDomegaA
                + 3. / 4. * mu2 / FastMath.PI * lambda * DmuDomegaA
                + (1. / 3. * B3 + 1. / 4. * mu3 / FastMath.PI) * DlambdaDomegaA
                + 1. / 4. * B2 * mu * b2 * DmuDomegaA
                + 1. / 8. * mu2 * B2 * Db2DomegaA
                + (1. / 6. * B3 - 5. / 12. * mu3 / FastMath.PI) * pc * DmuDomegaA
                - (1. / 6. * B3 * mu - 5. / 48. * mu4 / FastMath.PI) * pc);
        System.out.println("Da0DomegaA " + Da0DomegaA);
        System.out.println("Da0DomegaN " + Da0DomegaN);
        assertEquals("Da0Domega", Da0DomegaA, Da0DomegaN, tolA);

        // a1 derivatives ------------------------------------------------------
        // a1 value
        System.out.println();
        final double a1bis = 2 * mu / (B4 - 1. / 2. * mu2 * B2)
                * ((4. / 3. * B3 + 1. / 3. * mu3 / FastMath.PI) * theta0
                + B4 * thetaTW
                + (B2 - 1. / 4. * mu2) * lambda
                - 1. / 3. * B3 * b2
                + (1. / 2. * B4 / mu + 5. / 48. * mu3) * pc
                - 8 / gamma * qc / mu);
        System.out.println("a1 " + a1bis);
        System.out.println("a1 " + a1);
        // derivative with respect to u
        final double Da1DuA = (2 * B4 + mu2 * B2)
                / (2 * mu * (B4 - 1. / 2. * mu2 * B2)) * a1
                + 2 * mu / (B4 - 1. / 2. * mu2 * B2)
                * (mu2 / FastMath.PI * theta0 - 1. / 2. * mu * lambda
                + (B2 - 1. / 4. * mu2) * DlambdaDuA
                - 1. / 3. * B3 * Db2DuA
                + (-1. / 2. * B4 / mu2 + 5. / 16. * mu2) * pc
                + 8 / gamma * qc / mu2);
        System.out.println("Da1DuA " + Da1DuA);
        System.out.println("Da1DuN " + Da1DuN);
        assertEquals("Da1Du", Da1DuA, Da1DuN, tolA);
        // derivative with respect to w
        final double Da1DwA = 2 * mu / (B4 - 1. / 2. * mu2 * B2)
                * ((B2 - 1. / 4. * mu2) * DlambdaDwA
                - 1. / 3. * B3 * Db2DwA);
        System.out.println("Da1DwA " + Da1DwA);
        System.out.println("Da1DwN " + Da1DwN);
        assertEquals("Da1Dw", Da1DwA, Da1DwN, tolA);
        // derivative with respect to q
        final double Da1DqA = 2 * mu / (B4 - 1. / 2. * mu2 * B2)
                * ((B2 - 1. / 4. * mu2) * DlambdaDqA
                - 1. / 3. * B3 * Db2DqA - 8 / gamma / mu);
        System.out.println("Da1DqA " + Da1DqA);
        System.out.println("Da1DqN " + Da1DqN);
        assertEquals("Da1Dq", Da1DqA, Da1DqN, tolA);
        // derivative with respect to p
        final double Da1DpA = 2 * mu / (B4 - 1. / 2. * mu2 * B2)
                * ((B2 - 1. / 4. * mu2) * DlambdaDpA
                - 1. / 3. * B3 * Db2DpA
                + (1. / 2. * B4 / mu + 5. / 48. * mu3));
        System.out.println("Da1DpA " + Da1DpA);
        System.out.println("Da1DpN " + Da1DpN);
        assertEquals("Da1Dp", Da1DpA, Da1DpN, tolA);
        // derivative with respect to omega
        final double Da1DomegaA = (2 * B4 + mu2 * B2)
                / (2 * mu * (B4 - 1. / 2. * mu2 * B2)) * a1 * DmuDomegaA
                + 2 * mu / (B4 - 1. / 2. * mu2 * B2)
                * (mu2 / FastMath.PI * theta0 * DmuDomegaA
                - 1. / 2. * mu * lambda * DmuDomegaA
                + (B2 - 1. / 4. * mu2) * DlambdaDomegaA
                - 1. / 3. * B3 * Db2DomegaA
                + (-1. / 2. * B4 / mu2 + 5. / 16. * mu2)
                * pc * DmuDomegaA + 8 / gamma * qc / mu2 * DmuDomegaA
                - (1. / 2. * B4 / mu + 5. / 48. * mu3) * pc
                + 8 / gamma * qc / mu);
        System.out.println("Da1DomegaA " + Da1DomegaA);
        System.out.println("Da1DomegaN " + Da1DomegaN);
        assertEquals("Da1Domega", Da1DomegaA, Da1DomegaN, tolA);

        // b1 derivatives ------------------------------------------------------
        // b1 value
        System.out.println();
        final double b1bis = 4 * mu / (B4 + 1. / 2. * mu2 * B2)
                * (-4 * pc / (gamma * mu) - 1. / 4. * B4 * qc / mu
                + (1. / 3. * B3 + 1. / 9. * mu3 / FastMath.PI) * a0
                + 1. / 6. * B3 * a2);
        System.out.println("b1 " + b1bis);
        System.out.println("b1 " + b1);
        // derivative with respect to u
        final double Db1DuA = (B4 - 1. / 2. * mu2 * B2)
                / (mu * (B4 + 1. / 2. * mu2 * B2)) * b1
                + 4 * mu / (B4 + 1. / 2. * mu2 * B2)
                * (4 / gamma * pc / mu2 + 1. / 4. * B4 * qc / mu2
                + 1. / 3. * mu2 / FastMath.PI * a0
                + (1. / 3. * B3 + 1. / 9. * mu3 / FastMath.PI) * Da0DuA
                + 1. / 6. * B3 * Da2DuA);
        System.out.println("Db1DuA " + Db1DuA);
        System.out.println("Db1DuN " + Db1DuN);
        assertEquals("Db1Du", Db1DuA, Db1DuN, tolA);
        // derivative with respect to w
        final double Db1DwA = 4 * mu / (B4 + 1. / 2. * mu2 * B2)
                * (1. / 6. * B3 * Da2DwA
                + (1. / 3. * B3 + 1. / 9. * mu3 / FastMath.PI) * Da0DwA);
        System.out.println("Db1DwA " + Db1DwA);
        System.out.println("Db1DwN " + Db1DwN);
        assertEquals("Db1Dw", Db1DwA, Db1DwN, tolA);
        // derivative with respect to q
        final double Db1DqA = 4 * mu / (B4 + 1. / 2. * mu2 * B2)
                * (1. / 6. * B3 * Da2DqA
                + (1. / 3. * B3 + 1. / 9. * mu3 / FastMath.PI) * Da0DqA
                - 1. / 4. * B4 / mu);
        System.out.println("Db1DqA " + Db1DqA);
        System.out.println("Db1DqN " + Db1DqN);
        assertEquals("Db1Dq", Db1DqA, Db1DqN, tolA);
        // derivative with respect to p
        final double Db1DpA = 4 * mu / (B4 + 1. / 2. * mu2 * B2)
                * (1. / 6. * B3 * Da2DpA
                + (1. / 3. * B3 + 1. / 9. * mu3 / FastMath.PI) * Da0DpA
                - 4 / (gamma * mu));
        System.out.println("Db1DpA " + Db1DpA);
        System.out.println("Db1DpN " + Db1DpN);
        assertEquals("Db1Dp", Db1DpA, Db1DpN, tolA);
        // derivative with respect to omega
        final double Db1DomegaA = (B4 - 1. / 2. * mu2 * B2)
                / (mu * (B4 + 1. / 2. * mu2 * B2)) * b1 * DmuDomegaA
                + 4 * mu / (B4 + 1. / 2. * mu2 * B2)
                * (4 / gamma * pc / mu2 * DmuDomegaA
                + 4 * pc / (gamma * mu)
                + 1. / 4. * B4 * qc / mu2 * DmuDomegaA
                + 1. / 4. * B4 * qc / mu
                + 1. / 3. * mu2 / FastMath.PI * a0 * DmuDomegaA
                + (1. / 3. * B3 + 1. / 9. * mu3 / FastMath.PI) * Da0DomegaA
                + 1. / 6. * B3 * Da2DomegaA);
        System.out.println("Db1DomegaA " + Db1DomegaA);
        System.out.println("Db1DomegaN " + Db1DomegaN);
        assertEquals("Db1Domega", Db1DomegaA, Db1DomegaN, tolA);

        // CT derivatives ------------------------------------------------------
        // CTvalue
        System.out.println();
        final double CTbis = a / 2
                * ((1. / 4. * B2 * mu - 1. / 16. * mu3) * pc
                + 1. / 8. * mu3 * a1 + 1. / 4. * B * mu2 * b2
                + (1. / 2. * B2 + 1. / 4. * mu2) * lambda
                + (-4. / 9. * mu3 / FastMath.PI + 1. / 3. * B3
                + 1. / 2. * B * mu2) * theta0
                + (1. / 4. * mu2 * B2 + 1. / 4. * B4
                - 1. / 32. * mu4) * thetaTW);
        System.out.println("CT " + CTbis);
        System.out.println("CT " + CT);
        // derivative with respect to u
        final double DCTDuA = a / 2
                * ((1. / 4. * B2 - 3. / 16. * mu2) * pc
                + 3. / 8. * mu2 * a1 + 1. / 8. * mu3 * Da1DuA
                + 1. / 2. * B * mu * b2 + 1. / 4. * B * mu2 * Db2DuA
                + 1. / 2. * mu * lambda + (1. / 2. * B2 + 1. / 4. * mu2) * DlambdaDuA
                + (-4. / 3. * mu2 / FastMath.PI + B * mu) * theta0
                + (1. / 2. * B2 * mu - 1. / 8 * mu3) * thetaTW);
        System.out.println("DCTDuA " + DCTDuA);
        System.out.println("DCTDuN " + DCTDuN);
        assertEquals("DCTDu", DCTDuA, DCTDuN, tolA);
        // derivative with respect to w
        final double DCTDwA = a / 2
                * (1. / 8. * mu3 * Da1DwA + 1. / 4. * B * mu2 * Db2DwA
                + (1. / 2. * B2 + 1. / 4. * mu2) * DlambdaDwA);
        System.out.println("DCTDwA " + DCTDwA);
        System.out.println("DCTDwN " + DCTDwN);
        assertEquals("DCTDw", DCTDwA, DCTDwN, tolA);
        // derivative with respect to q
        final double DCTDqA = a / 2
                * (1. / 8. * mu3 * Da1DqA + 1. / 4. * B * mu2 * Db2DqA
                + (1. / 2. * B2 + 1. / 4. * mu2) * DlambdaDqA);
        System.out.println("DCTDqA " + DCTDqA);
        System.out.println("DCTDqN " + DCTDqN);
        assertEquals("DCTDq", DCTDqA, DCTDqN, tolA);
        // derivative with respect to p
        final double DCTDpA = a / 2
                * ((1. / 4. * B2 * mu - 1. / 16. * mu3)
                + 1. / 8. * mu3 * Da1DpA
                + 1. / 4. * B * mu2 * Db2DpA
                + (1. / 2. * B2 + 1. / 4. * mu2) * DlambdaDpA);
        System.out.println("DCTDpA " + DCTDpA);
        System.out.println("DCTDpN " + DCTDpN);
        assertEquals("DCTDp", DCTDpA, DCTDpN, tolA);
        // derivative with respect to omega
        final double DCTDomegaA = a / 2
                * ((1. / 4. * B2 - 3. / 16. * mu2) * pc * DmuDomegaA
                - (1. / 4. * B2 * mu - 1. / 16. * mu3) * pc
                + 3. / 8. * mu2 * a1 * DmuDomegaA + 1. / 8. * mu3 * Da1DomegaA
                + 1. / 2. * B * mu * b2 * DmuDomegaA + 1. / 4. * B * mu2 * Db2DomegaA
                + 1. / 2. * mu * lambda * DmuDomegaA + (1. / 2. * B2
                + 1. / 4. * mu2) * DlambdaDomegaA
                + (-4. / 3. * mu2 / FastMath.PI + B * mu) * theta0 * DmuDomegaA
                + (1. / 2. * B2 * mu - 1. / 8 * mu3) * thetaTW * DmuDomegaA);
        System.out.println("DCTDomegaA " + DCTDomegaA);
        System.out.println("DCTDomegaN " + DCTDomegaN);
        assertEquals("DCTDomega", DCTDomegaA, DCTDomegaN, tolA);

        // CH derivatives ------------------------------------------------------
        // CHvalue
        System.out.println();
        final double CHbis = 1. / 4. * delta * mu + a / 2
                * ((-1. / 2. * B2 * lambda - 1. / 8. * B4 * thetaTW
                - 1. / 6. * B3 * theta0 + 5. / 12. * B3 * b2
                - 1. / 16. * B2 * mu * a1) * pc
                + (-1. / 6. * B3 * a0 + 5. / 12. * B3 * a2
                - 1. / 16. * B2 * mu * b1) * qc
                + (3. / 4. * B2 * a1 - 1. / 4. * B2 * mu * thetaTW
                - 1. / 2. * B * theta0 * mu - 1. / 4. * B * mu * b2) * lambda
                + (1. / 3. * B3 * a1 + 3. / 8. * B2 * mu * b2) * theta0
                + (1. / 4. * B4 * a1 + 1. / 4. * B3 * mu * b2) * thetaTW
                + (-1. / 6. * B3 * b1 - 1. / 2. * B2 * mu * a2) * a0
                + 1. / 4. * B2 * mu * a0 * a0
                + -1. / 4. * B3 * b2 * a1 + 1. / 4. * B2 * mu * a1 * a1
                + 1. / 4. * B3 * b1 * a2);
        System.out.println("CH " + CHbis);
        System.out.println("CH " + CH);
        // derivative with respect to u
        final double DCHDuA = 1. / 4. * delta + a / 2
                * ((-1. / 2. * B2 * DlambdaDuA + 5. / 12. * B3 * Db2DuA
                - 1. / 16. * B2 * a1 - 1. / 16. * B2 * mu * Da1DuA) * pc
                + (-1. / 6. * B3 * Da0DuA + 5. / 12. * B3 * Da2DuA
                - 1. / 16. * B2 * b1 - 1. / 16. * B2 * mu * Db1DuA) * qc
                + (3. / 4. * B2 * Da1DuA - 1. / 4. * B2 * thetaTW
                - 1. / 2. * B * theta0 - 1. / 4. * B * b2
                - 1. / 4. * B * mu * Db2DuA) * lambda
                + (3. / 4. * B2 * a1 - 1. / 4. * B2 * mu * thetaTW
                - 1. / 2. * B * theta0 * mu - 1. / 4. * B * mu * b2) * DlambdaDuA
                + (1. / 3. * B3 * Da1DuA + 3. / 8. * B2 * b2
                + 3. / 8. * B2 * mu * Db2DuA) * theta0
                + (1. / 4. * B4 * Da1DuA + 1. / 4. * B3 * b2
                + 1. / 4. * B3 * mu * Db2DuA) * thetaTW
                + (-1. / 6. * B3 * Db1DuA - 1. / 2. * B2 * a2
                - 1. / 2. * B2 * mu * Da2DuA) * a0
                + (-1. / 6. * B3 * b1 - 1. / 2. * B2 * mu * a2) * Da0DuA
                + 1. / 4. * B2 * a0 * a0 + 1. / 2. * B2 * mu * a0 * Da0DuA
                - 1. / 4. * B3 * b2 * Da1DuA - 1. / 4 * B3 * a1 * Db2DuA
                + 1. / 4. * B2 * a1 * a1 + 1. / 2. * B2 * mu * a1 * Da1DuA
                + 1. / 4. * B3 * b1 * Da2DuA + 1. / 4. * B3 * a2 * Db1DuA);
        System.out.println("DCHDuA " + DCHDuA);
        System.out.println("DCHDuN " + DCHDuN);
        assertEquals("DCHDu", DCHDuA, DCHDuN, tolA);
        // derivative with respect to w
        final double DCHDwA = a / 2
                * ((-1. / 2. * B2 * DlambdaDwA + 5. / 12. * B3 * Db2DwA
                - 1. / 16. * B2 * mu * Da1DwA) * pc
                + (-1. / 6. * B3 * Da0DwA + 5. / 12. * B3 * Da2DwA
                - 1. / 16. * B2 * mu * Db1DwA) * qc
                + (3. / 4. * B2 * Da1DwA - 1. / 4. * B * mu * Db2DwA) * lambda
                + (3. / 4. * B2 * a1 - 1. / 4. * B2 * mu * thetaTW
                - 1. / 2. * B * theta0 * mu - 1. / 4. * B * mu * b2) * DlambdaDwA
                + (1. / 3. * B3 * Da1DwA + 3. / 8. * B2 * mu * Db2DwA) * theta0
                + (1. / 4. * B4 * Da1DwA + 1. / 4. * B3 * mu * Db2DwA) * thetaTW
                + (-1. / 6. * B3 * Db1DwA - 1. / 2. * B2 * mu * Da2DwA) * a0
                + (-1. / 6. * B3 * b1 - 1. / 2. * B2 * mu * a2) * Da0DwA
                + 1. / 2. * B2 * mu * a0 * Da0DwA - 1. / 4. * B3 * b2 * Da1DwA
                - 1. / 4 * B3 * a1 * Db2DwA + 1. / 2. * B2 * mu * a1 * Da1DwA
                + 1. / 4. * B3 * b1 * Da2DwA + 1. / 4. * B3 * a2 * Db1DwA);
        System.out.println("DCHDwA " + DCHDwA);
        System.out.println("DCHDwA " + DCHDwN);
        assertEquals("DCHDw", DCHDwA, DCHDwN, tolA);
        // derivative with respect to q
        final double DCHDqA = a / 2
                * ((-1. / 6. * B3 * a0 + 5. / 12. * B3 * a2 - 1. / 16. * B2 * mu * b1)
                + (-1. / 2. * B2 * DlambdaDqA + 5. / 12. * B3 * Db2DqA
                - 1. / 16. * B2 * mu * Da1DqA) * pc
                + (-1. / 6. * B3 * Da0DqA + 5. / 12. * B3 * Da2DqA
                - 1. / 16. * B2 * mu * Db1DqA) * qc
                + (3. / 4. * B2 * Da1DqA - 1. / 4. * B * mu * Db2DqA) * lambda
                + (3. / 4. * B2 * a1 - 1. / 4. * B2 * mu * thetaTW
                - 1. / 2. * B * theta0 * mu - 1. / 4. * B * mu * b2) * DlambdaDqA
                + (1. / 3. * B3 * Da1DqA + 3. / 8. * B2 * mu * Db2DqA) * theta0
                + (1. / 4. * B4 * Da1DqA + 1. / 4. * B3 * mu * Db2DqA) * thetaTW
                + (-1. / 6. * B3 * Db1DqA - 1. / 2. * B2 * mu * Da2DqA) * a0
                + (-1. / 6. * B3 * b1 - 1. / 2. * B2 * mu * a2) * Da0DqA
                + 1. / 2. * B2 * mu * a0 * Da0DqA - 1. / 4. * B3 * b2 * Da1DqA
                - 1. / 4 * B3 * a1 * Db2DqA + 1. / 2. * B2 * mu * a1 * Da1DqA
                + 1. / 4. * B3 * b1 * Da2DqA + 1. / 4. * B3 * a2 * Db1DqA);
        System.out.println("DCHDqA " + DCHDqA);
        System.out.println("DCHDqN " + DCHDqN);
        assertEquals("DCHDq", DCHDqA, DCHDqN, tolA);
        // derivative with respect to p
        final double DCHDpA = a / 2
                * ((-1. / 2. * B2 * lambda - 1. / 8. * B4 * thetaTW - 1. / 6. * B3 * theta0
                + 5. / 12. * B3 * b2 - 1. / 16. * B2 * mu * a1)
                + (-1. / 2. * B2 * DlambdaDpA + 5. / 12. * B3 * Db2DpA
                - 1. / 16. * B2 * mu * Da1DpA) * pc
                + (-1. / 6. * B3 * Da0DpA + 5. / 12. * B3 * Da2DpA
                - 1. / 16. * B2 * mu * Db1DpA) * qc
                + (3. / 4. * B2 * Da1DpA - 1. / 4. * B * mu * Db2DpA) * lambda
                + (3. / 4. * B2 * a1 - 1. / 4. * B2 * mu * thetaTW
                - 1. / 2. * B * theta0 * mu - 1. / 4. * B * mu * b2) * DlambdaDpA
                + (1. / 3. * B3 * Da1DpA + 3. / 8. * B2 * mu * Db2DpA) * theta0
                + (1. / 4. * B4 * Da1DpA + 1. / 4. * B3 * mu * Db2DpA) * thetaTW
                + (-1. / 6. * B3 * Db1DpA - 1. / 2. * B2 * mu * Da2DpA) * a0
                + (-1. / 6. * B3 * b1 - 1. / 2. * B2 * mu * a2) * Da0DpA
                + 1. / 2. * B2 * mu * a0 * Da0DpA - 1. / 4. * B3 * b2 * Da1DpA
                - 1. / 4 * B3 * a1 * Db2DpA + 1. / 2. * B2 * mu * a1 * Da1DpA
                + 1. / 4. * B3 * b1 * Da2DpA + 1. / 4. * B3 * a2 * Db1DpA);
        System.out.println("DCHDpA " + DCHDpA);
        System.out.println("DCHDpN " + DCHDpN);
        assertEquals("DCHDp", DCHDpA, DCHDpN, tolA);
        // derivative with respect to omega
        final double DCHDomegaA = 1. / 4. * delta * DmuDomegaA + a / 2
                * ((-1. / 2. * B2 * DlambdaDomegaA + 5. / 12. * B3 * Db2DomegaA
                - 1. / 16. * B2 * a1 * DmuDomegaA - 1. / 16. * B2 * mu * Da1DomegaA) * pc
                + (1. / 2. * B2 * lambda + 1. / 8. * B4 * thetaTW
                + 1. / 6. * B3 * theta0 - 5. / 12. * B3 * b2
                + 1. / 16. * B2 * mu * a1) * pc
                + (-1. / 6. * B3 * Da0DomegaA + 5. / 12. * B3 * Da2DomegaA
                - 1. / 16. * B2 * b1 * DmuDomegaA - 1. / 16. * B2 * mu * Db1DomegaA) * qc
                + (1. / 6. * B3 * a0 - 5. / 12. * B3 * a2 + 1. / 16. * B2 * mu * b1) * qc
                + (3. / 4. * B2 * Da1DomegaA
                - (1. / 4. * B2 * thetaTW + 1. / 2. * B * theta0 + 1. / 4. * B * b2) * DmuDomegaA
                - 1. / 4. * B * mu * Db2DomegaA) * lambda
                + (3. / 4. * B2 * a1 - 1. / 4. * B2 * mu * thetaTW
                - 1. / 2. * B * theta0 * mu - 1. / 4. * B * mu * b2) * DlambdaDomegaA
                + (1. / 3. * B3 * Da1DomegaA + 3. / 8. * B2 * b2 * DmuDomegaA
                + 3. / 8. * B2 * mu * Db2DomegaA) * theta0
                + (1. / 4. * B4 * Da1DomegaA + 1. / 4. * B3 * b2 * DmuDomegaA
                + 1. / 4. * B3 * mu * Db2DomegaA) * thetaTW
                + (-1. / 6. * B3 * Db1DomegaA - 1. / 2. * B2 * a2 * DmuDomegaA
                - 1. / 2. * B2 * mu * Da2DomegaA) * a0
                + (-1. / 6. * B3 * b1 - 1. / 2. * B2 * mu * a2) * Da0DomegaA
                + 1. / 4. * B2 * a0 * a0 * DmuDomegaA + 1. / 2. * B2 * mu * a0 * Da0DomegaA
                - 1. / 4. * B3 * b2 * Da1DomegaA - 1. / 4 * B3 * a1 * Db2DomegaA
                + 1. / 4. * B2 * a1 * a1 * DmuDomegaA + 1. / 2. * B2 * mu * a1 * Da1DomegaA
                + 1. / 4. * B3 * b1 * Da2DomegaA + 1. / 4. * B3 * a2 * Db1DomegaA);
        System.out.println("DCHDomegaA " + DCHDomegaA);
        System.out.println("DCHDomegaN " + DCHDomegaN);
        assertEquals("DCHDomega", DCHDomegaA, DCHDomegaN, tolA);

        // CQ derivatives ------------------------------------------------------
        // CQvalue
        System.out.println();
        final double CQbis = 0;
        System.out.println("CQ " + CQbis);
        System.out.println("CQ " + CQ);
        // derivative with respect to u
        final double DCQDuA = 0;
        System.out.println("DCQDuA " + DCQDuA);
        System.out.println("DCQDuN " + DCTDuN);
        //assertEquals("DCQDu", DCQDuA, DCTDuN, tolA);
        // derivative with respect to w
        final double DCQDwA = 0;
        System.out.println("DCQDwA " + DCQDwA);
        System.out.println("DCQDwN " + DCQDwN);
        //assertEquals("DCQDw", DCQDwA, DCQDwN, tolA);
        // derivative with respect to q
        final double DCQDqA = 0;
        System.out.println("DCQDqA " + DCQDqA);
        System.out.println("DCQDqN " + DCQDqN);
        // assertEquals("DCQDq", DCQDqA, DCQDqN, tolA);
        // derivative with respect to p
        final double DCQDpA = 0;
        System.out.println("DCQDpA " + DCQDpA);
        System.out.println("DCQDpN " + DCQDpN);
        //assertEquals("DCQDp", DCQDpA, DCQDpN, tolA);
        // derivative with respect to omega
        final double DCQDomegaA = 0;
        System.out.println("DCQDomegaA " + DCQDomegaA);
        System.out.println("DCQDomegaN " + DCQDomegaN);
        //assertEquals("DCQDomega", DCQDomegaA, DCQDomegaN, tolA);

        System.out.println(rotor.getInducedTorque() / KQ);
        System.out.println(rotor.getProfileDragTorque() / KQ);
        System.out.println(-delta / 8 * (1 + mu2 - mu4 / 8));
        final double CHi = rotor.getInducedRearForce() / KF;
        System.out.println(lambda * CT + mu * CHi);

        // CL derivatives -----------------------------------------------------
        // Incidence sin and cos
        final double sinAlphaS = FastMath.sin(alphaS);
        final double cosAlphaS = FastMath.cos(alphaS);
        System.out.println();
        // derivative with respect to u
        final double DCLDuA = DCTDuA * cosAlphaS - CT * sinAlphaS * DalphaSDuA
                - DCHDuA * sinAlphaS - CH * cosAlphaS * DalphaSDuA;
        System.out.println("DCLDuA " + DCLDuA);
        System.out.println("DCLDuN " + DCLDuN);
        assertEquals("DCLDu", DCLDuA, DCLDuN, tolA);
        // derivative with respect to w
        final double DCLDwA = DCTDwA * cosAlphaS - CT * sinAlphaS * DalphaSDwA
                - DCHDwA * sinAlphaS - CH * cosAlphaS * DalphaSDwA;
        System.out.println("DCLDwA " + DCLDwA);
        System.out.println("DCLDwN " + DCLDwN);
        assertEquals("DCLDw", DCLDwA, DCLDwN, tolA);
        // derivative with respect to q
        final double DCLDqA = DCTDqA * cosAlphaS - DCHDqA * sinAlphaS;
        System.out.println("DCLDqA " + DCLDqA);
        System.out.println("DCLDqN " + DCLDqN);
        assertEquals("DCLDq", DCLDqA, DCLDqN, tolA);
        // derivative with respect to p
        final double DCLDpA = DCTDpA * cosAlphaS - DCHDpA * sinAlphaS;
        System.out.println("DCLDpA " + DCLDpA);
        System.out.println("DCLDpN " + DCLDpN);
        assertEquals("DCLDp", DCLDpA, DCLDpN, tolA);
        // derivative with respect to omega
        final double DCLDomegaA = DCTDomegaA * cosAlphaS - DCHDomegaA * sinAlphaS;
        System.out.println("DCLDomegaA " + DCLDomegaA);
        System.out.println("DCLDomegaN " + DCLDomegaN);
        assertEquals("DCLDomega", DCLDomegaA, DCLDomegaN, tolA);
        
        // CD derivatives -----------------------------------------------------
        System.out.println();
        // derivative with respect to u
        final double DCDDuA = DCTDuA * sinAlphaS + CT * cosAlphaS * DalphaSDuA
                + DCHDuA * cosAlphaS - CH * sinAlphaS * DalphaSDuA;
        System.out.println("DCDDuA " + DCDDuA);
        System.out.println("DCDDuN " + DCDDuN);
        assertEquals("DCDDu", DCDDuA, DCDDuN, tolA);
        // derivative with respect to w
        final double DCDDwA = DCTDwA * sinAlphaS + CT * cosAlphaS * DalphaSDwA
                + DCHDwA * cosAlphaS - CH * sinAlphaS * DalphaSDwA;
        System.out.println("DCDDwA " + DCDDwA);
        System.out.println("DCDDwN " + DCDDwN);
        assertEquals("DCDDw", DCDDwA, DCDDwN, tolA);
        // derivative with respect to q
        final double DCDDqA = DCTDqA * sinAlphaS + DCHDqA * cosAlphaS;
        System.out.println("DCDDqA " + DCDDqA);
        System.out.println("DCDDqN " + DCDDqN);
        assertEquals("DCDDq", DCDDqA, DCDDqN, tolA);
        // derivative with respect to p
        final double DCDDpA = DCTDpA * sinAlphaS + DCHDpA * cosAlphaS;
        System.out.println("DCDDpA " + DCDDpA);
        System.out.println("DCDDpN " + DCDDpN);
        assertEquals("DCDDp", DCDDpA, DCDDpN, tolA);
        // derivative with respect to omega
        final double DCDDomegaA = DCTDomegaA * sinAlphaS + DCHDomegaA * cosAlphaS;
        System.out.println("DCDDomegaA " + DCDDomegaA);
        System.out.println("DCDDomegaN " + DCDDomegaN);
        assertEquals("DCDDomega", DCDDomegaA, DCDDomegaN, tolA);

        // Derivatives of X and Z forces in fuselage frame ---------------------
        // Compute up and wp velocity component in fuselage frame
        final double up = U * FastMath.cos(alphaF);
        final double wp = U * FastMath.sin(alphaF);
        final double step = 1e-7;

        // Derivatives with respect to u and w (coordinates of U in rotor frame)
        final double DXDuA = derivativesWRTfuselage[0][0];
        final double DXDwA = derivativesWRTfuselage[0][1];
        final double DXDqA = derivativesWRTfuselage[0][2];
        final double DXDpA = derivativesWRTfuselage[0][3];
        final double DXDomegaA = derivativesWRTfuselage[0][4];
        final double DYDuA = derivativesWRTfuselage[1][0];
        final double DYDwA = derivativesWRTfuselage[1][1];
        final double DYDqA = derivativesWRTfuselage[1][2];
        final double DYDpA = derivativesWRTfuselage[1][3];
        final double DYDomegaA = derivativesWRTfuselage[1][4];
        final double DZDuA = derivativesWRTfuselage[2][0];
        final double DZDwA = derivativesWRTfuselage[2][1];
        final double DZDqA = derivativesWRTfuselage[2][2];
        final double DZDpA = derivativesWRTfuselage[2][3];
        final double DZDomegaA = derivativesWRTfuselage[2][4];

        // Derivative with respect to up ---------------------------------------
        double upd = up - step;
        double Upd = FastMath.sqrt(upd * upd + wp * wp);
        double alphaSd = FastMath.atan(wp / upd) + alphaS - alphaF;
        rotor.computeState(Upd, alphaSd, theta0, q, p, omega, atmosphere);
        double L2 = rotor.getLift();
        double D2 = rotor.getDrag();
        double X2 = 1 / Upd * (-upd * D2 + wp * L2);
        double Y2 = rotor.getLateralForce();
        double Z2 = 1 / Upd * (-wp * D2 - upd * L2);

        upd = up + step;
        Upd = FastMath.sqrt(upd * upd + wp * wp);
        alphaSd = FastMath.atan(wp / upd) + alphaS - alphaF;
        rotor.computeState(Upd, alphaSd, theta0, q, p, omega, atmosphere);
        L2 = rotor.getLift();
        D2 = rotor.getDrag();
        double DXDuN = (1 / Upd * (-upd * D2 + wp * L2) - X2) / (2 * step);
        double DYDuN = (rotor.getLateralForce() - Y2) / (2 * step);
        double DZDuN = (1 / Upd * (-wp * D2 - upd * L2) - Z2) / (2 * step);

        System.out.println();
        System.out.println("DXDuA " + DXDuA);
        System.out.println("DXDuN " + DXDuN);
        assertEquals("DXDu", DXDuA, DXDuN, tolF);
        System.out.println("DYDuA " + DYDuA);
        System.out.println("DYDuN " + DYDuN);
        assertEquals("DYDu", DYDuA, DYDuN, tolF);
        System.out.println("DZDuA " + DZDuA);
        System.out.println("DZDuN " + DZDuN);
        assertEquals("DZDu", DZDuA, DZDuN, tolF);

        // Derivative with respect to w ----------------------------------------
        double wpd = wp - step;
        Upd = FastMath.sqrt(up * up + wpd * wpd);
        alphaSd = FastMath.atan(wpd / up) + alphaS - alphaF;
        rotor.computeState(Upd, alphaSd, theta0, q, p, omega, atmosphere);
        L2 = rotor.getLift();
        D2 = rotor.getDrag();
        X2 = 1 / Upd * (-up * D2 + wpd * L2);
        Y2 = rotor.getLateralForce();
        Z2 = 1 / Upd * (-wpd * D2 - up * L2);

        wpd = wp + step;
        Upd = FastMath.sqrt(up * up + wpd * wpd);
        alphaSd = FastMath.atan(wpd / up) + alphaS - alphaF;
        rotor.computeState(Upd, alphaSd, theta0, q, p, omega, atmosphere);
        L2 = rotor.getLift();
        D2 = rotor.getDrag();
        double DXDwN = (1 / Upd * (-up * D2 + wpd * L2) - X2) / (2 * step);
        double DYDwN = (rotor.getLateralForce() - Y2) / (2 * step);
        double DZDwN = (1 / Upd * (-wpd * D2 - up * L2) - Z2) / (2 * step);

        System.out.println();
        System.out.println("DXDwA " + DXDwA);
        System.out.println("DXDwN " + DXDwN);
        assertEquals("DXDw", DXDwA, DXDwN, tolF);
        System.out.println("DYDwA " + DYDwA);
        System.out.println("DYDwN " + DYDwN);
        assertEquals("DTDw", DYDwA, DYDwN, tolF);
        System.out.println("DZDwA " + DZDwA);
        System.out.println("DZDwN " + DZDwN);
        assertEquals("DZDw", DZDwA, DZDwN, tolF);

        // Derivative with respect to q ----------------------------------------
        double q2 = q - step;
        rotor.computeState(U, alphaS, theta0, q2, p, omega, atmosphere);
        L2 = rotor.getLift();
        D2 = rotor.getDrag();
        X2 = 1 / U * (-up * D2 + wp * L2);
        Y2 = rotor.getLateralForce();
        Z2 = 1 / U * (-wp * D2 - up * L2);

        q2 = q + step;
        rotor.computeState(U, alphaS, theta0, q2, p, omega, atmosphere);
        L2 = rotor.getLift();
        D2 = rotor.getDrag();
        double DXDqN = (1 / U * (-up * D2 + wp * L2) - X2) / (2 * step);
        double DYDqN = (rotor.getLateralForce() - Y2) / (2 * step);
        double DZDqN = (1 / U * (-wp * D2 - up * L2) - Z2) / (2 * step);

        System.out.println();
        System.out.println("DXDqA " + DXDqA);
        System.out.println("DXDqN " + DXDqN);
        assertEquals("DXDq", DXDqA, DXDqN, tolF);
        System.out.println("DYDqA " + DYDqA);
        System.out.println("DYDqN " + DYDqN);
        assertEquals("DYDq", DYDqA, DYDqN, tolF);
        System.out.println("DZDqA " + DZDqA);
        System.out.println("DZDqN " + DZDqN);
        assertEquals("DZDq", DZDqA, DZDqN, tolF);

        // Derivative with respect to p ----------------------------------------
        double p2 = p - step;
        rotor.computeState(U, alphaS, theta0, q, p2, omega, atmosphere);
        L2 = rotor.getLift();
        D2 = rotor.getDrag();
        X2 = 1 / U * (-up * D2 + wp * L2);
        Y2 = rotor.getLateralForce();
        Z2 = 1 / U * (-wp * D2 - up * L2);

        p2 = p + step;
        rotor.computeState(U, alphaS, theta0, q, p2, omega, atmosphere);
        L2 = rotor.getLift();
        D2 = rotor.getDrag();
        double DXDpN = (1 / U * (-up * D2 + wp * L2) - X2) / (2 * step);
        double DYDpN = (rotor.getLateralForce() - Y2) / (2 * step);
        double DZDpN = (1 / U * (-wp * D2 - up * L2) - Z2) / (2 * step);

        System.out.println();
        System.out.println("DXDpA " + DXDpA);
        System.out.println("DXDpN " + DXDpN);
        assertEquals("DXDp", DXDpA, DXDpN, tolF);
        System.out.println("DYDpA " + DYDpA);
        System.out.println("DYDpN " + DYDpN);
        assertEquals("DYDp", DYDpA, DYDpN, tolF);
        System.out.println("DZDpA " + DZDpA);
        System.out.println("DZDpN " + DZDpN);
        assertEquals("DZDp", DZDpA, DZDpN, tolF);

        // Derivative with respect to omega ------------------------------------
        double omega2 = omega - step;
        rotor.computeState(U, alphaS, theta0, q, p, omega2, atmosphere);
        L2 = rotor.getLift();
        D2 = rotor.getDrag();
        X2 = 1 / U * (-up * D2 + wp * L2);
        Y2 = rotor.getLateralForce();
        Z2 = 1 / U * (-wp * D2 - up * L2);

        omega2 = omega + step;
        rotor.computeState(U, alphaS, theta0, q, p, omega2, atmosphere);
        L2 = rotor.getLift();
        D2 = rotor.getDrag();
        double DXDomegaN = (1 / U * (-up * D2 + wp * L2) - X2) / (2 * step);
        double DYDomegaN = (rotor.getLateralForce() - Y2) / (2 * step);
        double DZDomegaN = (1 / U * (-wp * D2 - up * L2) - Z2) / (2 * step);

        System.out.println();
        System.out.println("DXDomegaA " + DXDomegaA);
        System.out.println("DXDomegaN " + DXDomegaN);
        assertEquals("DXDomega", DXDomegaA, DXDomegaN, tolF);
        System.out.println("DYDomegaA " + DYDomegaA);
        System.out.println("DYDomegaN " + DYDomegaN);
        assertEquals("DYDomega", DYDomegaA, DYDomegaN, tolF);
        System.out.println("DZDomegaA " + DZDomegaA);
        System.out.println("DZDomegaN " + DZDomegaN);
        assertEquals("DZDomega", DZDomegaA, DZDomegaN, tolF);

    }
}
